#!/usr/bin/env python

from cogent.util.unit_test import TestCase, main
from cogent.core.info import Info
from cogent.parse.ilm import ilm_parser

__author__ = "Shandy Wikman"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__contributors__ = ["Shandy Wikman"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Shandy Wikman"
__email__ = "ens01svn@cs.umu.se"
__status__ = "Development"

class IlmParserTest(TestCase):
    """Provides tests for ILM RNA secondary structure format parsers"""

    def setUp(self):
        """Setup function"""
        
        #output
        self.ilm_out = ILM
        #expected
        self.ilm_exp = [[(0,13),(1,12),(2,11),(6,7)]]

        
    def test_ilm_output(self):
        """Test for ilm format"""
        
        obs = ilm_parser(self.ilm_out)
        self.assertEqual(obs,self.ilm_exp)

ILM = ['\n', 'Final Matching:\n', '1 14\n', '2 13\n', '3 12\n', '4 0\n', 
'5 0\n', '6 0\n', '7 8\n', '8 7\n', '9 0\n', '10 0\n', '11 0\n', '12 3\n', 
'13 2\n', '14 1\n']

if __name__ == '__main__':
    main()



#!/bin/env python
# file text_tree.py
"""Simple base text representation of phylo tree."""

__author__ = "Micah Hamady"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Micah Hamady"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Micah Hamady"
__email__ = "hamady@colorado.edu"
__status__ = "Prototype"

class TextNode(object):
    """ Helper to display text phyolo tree """
    def __init__(self, branch_length, name):
        """Set up defaults"""
        self.BranchLength = 0.0
        if branch_length:
            self.BranchLength = branch_length

        self.Name = name
        self.Printed = False
        self.NumChildren = 0
        self.NumChildrenPrinted = 0
        self.LastChild = None

    def display(self, max_dist, scale):
        """Display current node - should refactor this"""
        delimiter = "-"
        pdelimiter = "+"

        if self.Printed:
            delimiter = " "
            pdelimiter = "|"

        # compute number of children last child has
        last_ct = 0
        if self.LastChild:
            last_ct = self.LastChild.NumChildren

        # update values
        self.Printed = True
        self.NumChildrenPrinted += 1
        print_ct = self.NumChildren - last_ct

        if self.NumChildrenPrinted == (print_ct + 1): # or \
            delimiter = " "
            pdelimiter = "+"
        elif self.NumChildrenPrinted > (print_ct + 1):
            pdelimiter = " "
            delimiter = " "
        if (self.NumChildren == self.NumChildrenPrinted and self.NumChildren == print_ct): 
            delimiter = " "
            pdelimiter = "+"
 
        # check if leaf
        dout = ""
        if self.Name:
            dout = "@@%s" % self.Name 
            pdelimiter = ">"
            delimiter = "-"

        return (int(self.BranchLength) - 1) * delimiter + pdelimiter + dout

def process_nodes(all_nodes, cur_nodes, cur_node, parent):
    """ Recursively process nodes """
    # make current node
    pn = TextNode(cur_node.Length, cur_node.Name)

    # set current node as last child of parent (last one wins)
    if parent:
        parent.LastChild = pn

    # handle terminal node
    if not cur_node.Children:
        all_nodes.append(cur_nodes + [pn])

    # internal node
    else:
        cur_nodes.append(pn)
        pn.NumChildren = 0 
        for child in cur_node.Children:
            if child.Children:
                pn.NumChildren +=  len(child.tips())
            else:
                pn.NumChildren += 1 
        for child in cur_node.Children: 
            process_nodes(all_nodes, cur_nodes, child, pn)
        cur_nodes.pop()

def generate_nodes(tree, max_dist, scale):
    """Iterate over list of TextNodes to display """ 
    all_nodes = []
    cur_nodes = []
    
    # generate list of lists of TextNodes
    process_nodes(all_nodes, cur_nodes, tree, None)

    # process each list of TextNodes
    for node_list in all_nodes:

        # generate text string and node key 
        ticks, node_key =  ''.join([x.display(max_dist, scale) for x in node_list]).split("@@")
        
        # compute distances
        dist = sum([x.BranchLength for x in node_list])
        #scaled_dist = sum([x.ScaledBranchLength for x in node_list])
        scaled_dist = sum([x.BranchLength for x in node_list])
        branch_len = node_list[-1].BranchLength

        # yield each node 
        yield ticks, node_key, dist, scaled_dist, branch_len




#!/usr/bin/env python
"""Tests of the AAIndex parser.
"""

from cogent.util.unit_test import TestCase, main
from cogent.parse.aaindex import AAIndex1Parser, AAIndex2Parser,\
AAIndexRecord, AAIndex1Record, AAIndex2Record, AAIndex1FromFiles,\
AAIndex2FromFiles

__author__ = "Greg Caporaso"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Greg Caporaso", "Rob Knight"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Greg Caporaso"
__email__ = "caporaso@colorado.edu"
__status__ = "Production"

class test_aaindex1_parser(TestCase):
    """ Tests aindex1_parser class """
    def setUp(self):
        """ Setup some variables """
        self._fake_file = list(fake_file_aaindex1.split('\n'))

        self.AAIndexObjects = AAIndex1FromFiles(self._fake_file)

    def test_init(self):
        """ AAI1: Test that init run w/o error """
        aa1p = AAIndex1Parser()
        
    def test_read_file_as_list(self):
        """AAI1: Test that a file is correctly opened as a list """
        aap = AAIndex1Parser()
        AAIndexObjects = aap(self._fake_file)

    def test_correct_num_of_records(self):
        """AAI1: Test that one object is created per record """
        self.assertEqual(6, len(self.AAIndexObjects))
       
    def test_ID_entries(self):
        """ AAI1: Test ID Entries """
        self.assertEqual(self.AAIndexObjects['ANDN920101'].ID, 'ANDN920101')
        self.assertEqual(self.AAIndexObjects['ARGP820103'].ID, 'ARGP820103')
        self.assertEqual(self.AAIndexObjects['JURD980101'].ID, 'JURD980101')

    def test_single_line_Description_entries(self):
        """ AAI1: Test Single Line Description Entries """
        self.assertEqual(self.AAIndexObjects['ANDN920101'].Description,\
        'alpha-CH chemical shifts (Andersen et al., 1992)')
        self.assertEqual(self.AAIndexObjects['ARGP820103'].Description,\
        'Membrane-buried preference parameters (Argos et al., 1982)')

    def test_multi_line_Description_entries(self):
        """ AAI1: Test Multi Line Description Entries """        
        self.assertEqual(self.AAIndexObjects['JURD980101'].Description,\
        'Modified Kyte-Doolittle hydrophobicity scale (Juretic et al., 1998)')
        
    def test_LITDB_entries(self):
        """ AAI1: Test LITDB Entries """
        self.assertEqual(self.AAIndexObjects['ANDN920101'].LITDBEntryNum,\
        'LIT:1810048b PMID:1575719')
        self.assertEqual(self.AAIndexObjects['ARGP820103'].LITDBEntryNum,\
        'LIT:0901079b PMID:7151796')
        self.assertEqual(self.AAIndexObjects['JURD980101'].LITDBEntryNum,\
        '')

    def test_Authors_entries(self):
        """ AAI1: Test Authors Entries """
        self.assertEqual(self.AAIndexObjects['ANDN920101'].Authors,\
        'Andersen, N.H., Cao, B. and Chen, C.')
        self.assertEqual(self.AAIndexObjects['ARGP820103'].Authors,\
        'Argos, P., Rao, J.K.M. and Hargrave, P.A.')
        self.assertEqual(self.AAIndexObjects['JURD980101'].Authors,\
        'Juretic, D., Lucic, B., Zucic, D. and Trinajstic, N.')                

    def test_mult_line_Title_entries(self):
        """ AAI1: Test Multi Line Title Entries """
        self.assertEqual(self.AAIndexObjects['ANDN920101'].Title,\
        'Peptide/protein structure analysis using the chemical shift index ' +\
        'method: upfield alpha-CH values reveal dynamic helices and aL sites')
        self.assertEqual(self.AAIndexObjects['JURD980101'].Title,\
        'Protein transmembrane structure: recognition and prediction by ' +\
        'using hydrophobicity scales through preference functions')

    def test_sing_line_Title_entries(self):
        """ AAI1: Test Single Line Title Entries """
        self.assertEqual(self.AAIndexObjects['ARGP820103'].Title,\
        'Structural prediction of membrane-bound proteins')     
        
    def test_Citation_entries(self):
        """ AAI1: Test Citation Entries """
        self.assertEqual(self.AAIndexObjects['ANDN920101'].Citation,\
        'Biochem. and Biophys. Res. Comm. 184, 1008-1014 (1992)')
        self.assertEqual(self.AAIndexObjects['ARGP820103'].Citation,\
        'Eur. J. Biochem. 128, 565-575 (1982)')
        self.assertEqual(self.AAIndexObjects['JURD980101'].Citation,\
        'Theoretical and Computational Chemistry, 5, 405-445 (1998)')
         
    def test_Comments_entries(self):
        """ AAI1: Test Comments Entries """
        self.assertEqual(self.AAIndexObjects['ANDN920101'].Comments,\
        '')
        self.assertEqual(self.AAIndexObjects['ARGP820103'].Comments,\
        '')
        self.assertEqual(self.AAIndexObjects['JURD980101'].Comments,\
        '')
        self.assertEqual(self.AAIndexObjects['TSAJ990102'].Comments,\
        '(Cyh 113.7)')

    def test_single_line_Correlating_entries(self):
         """ AAI1: Test single line Correlating Entries """
         self.assertEqual(self.AAIndexObjects['ANDN920101'].\
         Correlating['BUNA790102'], 0.949)

    def test_empty_Correlating_entries(self):
        """ AAI1: Test empty Correlating Entries """       
        self.assertEqual(self.AAIndexObjects['WILM950104'].Correlating, {})

    def test_multi_line_Correlating_entries(self):
         """ AAI1: Test multi line Correlating Entries """
         self.assertEqual(self.AAIndexObjects['ARGP820103'].\
         Correlating['ARGP820102'], 0.961)
         self.assertEqual(self.AAIndexObjects['ARGP820103'].\
         Correlating['MIYS850101'], 0.822)
         self.assertEqual(self.AAIndexObjects['ARGP820103'].\
         Correlating['JURD980101'], 0.800)

         self.assertEqual(self.AAIndexObjects['JURD980101'].\
         Correlating['KYTJ820101'], 0.996)
         self.assertEqual(self.AAIndexObjects['JURD980101'].\
         Correlating['NADH010101'], 0.925)
         self.assertEqual(self.AAIndexObjects['JURD980101'].\
         Correlating['OOBM770101'], -0.903)
                                    
    def test_Data_entries(self):
        """ AAI1: Test Data Entries """
        self.assertEqual(self.AAIndexObjects['ANDN920101'].Data['A'],\
        4.35)
        self.assertEqual(self.AAIndexObjects['ANDN920101'].Data['Q'],\
        4.37)
        self.assertEqual(self.AAIndexObjects['ANDN920101'].Data['V'],\
        3.95)
        self.assertEqual(self.AAIndexObjects['ARGP820103'].Data['A'],\
        1.56)
        self.assertEqual(self.AAIndexObjects['ARGP820103'].Data['Q'],\
        0.51)
        self.assertEqual(self.AAIndexObjects['ARGP820103'].Data['V'],\
        1.14)
        self.assertEqual(self.AAIndexObjects['JURD980101'].Data['A'],\
        1.10)
        self.assertEqual(self.AAIndexObjects['JURD980101'].Data['Q'],\
        -3.68)
        self.assertEqual(self.AAIndexObjects['JURD980101'].Data['V'],\
        4.2)                  

                   
class test_aaindex2_parser(TestCase):
    def setUp(self):
        """ Setup some variables """
        self._fake_file = list(fake_file_aaindex2.split('\n'))
        self.AAIndexObjects = AAIndex2FromFiles(self._fake_file)

    def test_init(self):
        """ AAI2: Test that init run w/o error """
        aa2p = AAIndex2Parser()

    def test_read_file_as_list(self):
        """AAI1: Test that a file is correctly opened as a list """
        aap = AAIndex2Parser()
        AAIndexObjects = aap(self._fake_file)

    def test_correct_num_of_records(self):
        """AAI2: Test that one object is created per record """
        self.assertEqual(6, len(self.AAIndexObjects))

    def test_ID_entries(self):
        """ AAI2: Test ID Entries """
        self.assertEqual(self.AAIndexObjects['ALTS910101'].ID, 'ALTS910101')
        self.assertEqual(self.AAIndexObjects['BENS940103'].ID, 'BENS940103')
        self.assertEqual(self.AAIndexObjects['QUIB020101'].ID, 'QUIB020101')

    def test_Description_entries(self):
        """ AAI2: Test Description Entries """
        self.assertEqual(self.AAIndexObjects['ALTS910101'].Description,\
        'The PAM-120 matrix (Altschul, 1991)')
        self.assertEqual(self.AAIndexObjects['BENS940103'].Description,\
        'Log-odds scoring matrix collected in 74-100 PAM (Benner et al., '+\
        '1994)')
        self.assertEqual(self.AAIndexObjects['QUIB020101'].Description,\
        'STROMA score matrix for the alignment of known distant homologs ' +\
        '(Qian-Goldstein, 2002)')

    def test_LITDB_entries(self):
        """ AAI2: Test LITDB Entries """
        self.assertEqual(self.AAIndexObjects['ALTS910101'].LITDBEntryNum,\
        'LIT:1713145 PMID:2051488')
        self.assertEqual(self.AAIndexObjects['BENS940103'].LITDBEntryNum,\
        'LIT:2023094 PMID:7700864')
        self.assertEqual(self.AAIndexObjects['QUIB020101'].LITDBEntryNum,\
        'PMID:12211027')

    def test_Authors_entries(self):
        """ AAI2: Test Atuthor Entries """
        self.assertEqual(self.AAIndexObjects['ALTS910101'].Authors,\
        'Altschul, S.F.')
        self.assertEqual(self.AAIndexObjects['BENS940103'].Authors,\
        'Benner, S.A., Cohen, M.A. and Gonnet, G.H.')
        self.assertEqual(self.AAIndexObjects['QUIB020101'].Authors,\
        'Qian, B. and Goldstein, R.A.')

    def test_Title_entries(self):
        """ AAI2: Test Title Entries """
        self.assertEqual(self.AAIndexObjects['ALTS910101'].Title,\
        'Amino acid substitution matrices from an information theoretic ' +\
        'perspective')         
        self.assertEqual(self.AAIndexObjects['BENS940103'].Title,\
        'Amino acid substitution during functionally constrained divergent ' +\
        'evolution of protein sequences')
        self.assertEqual(self.AAIndexObjects['QUIB020101'].Title,\
        'Optimization of a new score function for the generation of '+\
        'accurate alignments')
        
    def test_Citation_entries(self):
        """ AAI2: Test citation entries """
        self.assertEqual(self.AAIndexObjects['ALTS910101'].Citation,\
        'J. Mol. Biol. 219, 555-565 (1991)')
        self.assertEqual(self.AAIndexObjects['BENS940103'].Citation,\
        'Protein Engineering 7, 1323-1332 (1994)')
        self.assertEqual(self.AAIndexObjects['QUIB020101'].Citation,\
        'Proteins. 48, 605-610 (2002)')

    def test_Comments_entries(self):
        """ AAI2: Tests null, single line, multi line comments """
        self.assertEqual(self.AAIndexObjects['ALTS910101'].Comments,\
        '')
        self.assertEqual(self.AAIndexObjects['BENS940103'].Comments,\
        'extrapolated to 250 PAM')
        self.assertEqual(self.AAIndexObjects['QUIB020101'].Comments,\
        '')
        self.assertEqual(self.AAIndexObjects['HENS920104'].Comments,\
        '#  Matrix made by matblas from blosum50.iij ' +
        '* #  BLOSUM Clustered Scoring Matrix in 1/3 Bit Units ' +
        '* #  Blocks Database = /data/blocks_5.0/blocks.dat ' +
        '* #  Cluster Percentage: >= 50 ' +
        '* #  Entropy =   0.4808, Expected =  -0.3573')
      
    def test_Data_entries_20x20_LTM(self):
         """ AAI2: correct data entries when 20x20 LTM"""
         self.assertEqual(self.AAIndexObjects['ALTS910101'].Data['A']['A'],\
         3.)
         self.assertEqual(self.AAIndexObjects['ALTS910101'].Data['Y']['R'],\
         -6.)
         self.assertEqual(self.AAIndexObjects['ALTS910101'].Data['V']['V'],\
         5.)
         self.assertEqual(self.AAIndexObjects['BENS940103'].Data['A']['A'],\
         2.4)
         self.assertEqual(self.AAIndexObjects['BENS940103'].Data['Y']['R'],\
         -2.0)
         self.assertEqual(self.AAIndexObjects['BENS940103'].Data['V']['V'],\
         3.4)
         self.assertEqual(self.AAIndexObjects['QUIB020101'].Data['A']['A'],\
         2.5)
         self.assertEqual(self.AAIndexObjects['QUIB020101'].Data['Y']['R'],\
         -0.9)
         self.assertEqual(self.AAIndexObjects['QUIB020101'].Data['V']['V'],\
         4.2)
         
    def test_Data_entries_20x20_Square(self):
        """ AAI2: correct data entries when 20x20 squ matrix """
        self.assertEqual(self.AAIndexObjects['HENS920104'].Data['V']['Y'],\
        -1)
        self.assertEqual(self.AAIndexObjects['HENS920104'].Data['Q']['A'],\
        -1)
        self.assertEqual(self.AAIndexObjects['HENS920104'].Data['N']['N'],\
        7)

    def test_Data_entries_with_abnormal_fields(self):
        """ AAI2: test correct data entries when more than std fields present

            Some entires in AAIndex2 have more that 20 fields, this tests that
            that data is corrected parsed and identified.
        """
        # There are no entries that fit this category that are square
        # matrices, which is all we are concerned with at this point,
        # so this method should just serve as a reminder to test this
        # when we begin parsing data other than square matrices.
        pass
        
    def test_Data_entries_21x21_LTM(self):
        """ AAI2: correct data entries when 21x21 LTM"""
        self.assertEqual(self.AAIndexObjects['KOSJ950101'].Data['-']['-'],\
        55.7)
        self.assertEqual(self.AAIndexObjects['KOSJ950101'].Data['Y']['-'],\
        0.3)
        self.assertEqual(self.AAIndexObjects['KOSJ950101'].Data['N']['R'],\
        3.0)

    def test_Data_entries_22x21_square(self):
        """ AAI2: correct data entries when 22x21 square matrix """
        # It's not really a sqaure matrix, but it's fully populated ...
        self.assertEqual(self.AAIndexObjects['OVEJ920102'].Data['J']['D'],\
        0.001)
        self.assertEqual(self.AAIndexObjects['OVEJ920102'].Data['-']['I'],\
        0.022)
        self.assertEqual(self.AAIndexObjects['OVEJ920102'].Data['D']['E'],\
        0.109)

        
class AAIndexRecordTests(TestCase):
    """ AAIR: Tests AAIndexRecord class """

    def setUp(self):
        self.id = "5"
        self.description = "Some Info"
        self.LITDB_entry_num = "25"
        self.authors = "Greg"
        self.title = "A test"
        self.citation = "something"
        self.comments = "This is a test, this is only a test"
        self.data = {}


class AAIndex1RecordTests(AAIndexRecordTests):
    """ AAIR1: Tests AAIndex1Records class """

    def setUp(self):
        AAIndexRecordTests.setUp(self)

        self.correlating = [0.987, 0.783, 1., 0]

        values = []
        keys = 'ARNDCQEGHILKMFPSTWYV'
        for i in range(20):
            values += [float(i) + 0.15]
        self.data = dict(zip(keys,values))

        self.aar = AAIndex1Record(self.id, self.description,\
                self.LITDB_entry_num, self.authors, self.title,\
                self.citation, self.comments, self.correlating, self.data)

    def test_init(self):
        """ AAIR1: Tests init method returns with no errors"""

        test_aar = AAIndex1Record(self.id, self.description,\
                self.LITDB_entry_num, self.authors, self.title,\
                self.citation, self.comments, self.correlating, self.data)

    def test_general_init_data(self):
        """ AAIR1: Tests init correctly initializes data"""

        self.assertEqual(self.aar.ID, str(self.id))
        self.assertEqual(self.aar.Description, str(self.description))
        self.assertEqual(self.aar.LITDBEntryNum,\
                str(self.LITDB_entry_num))
        self.assertEqual(self.aar.Authors, str(self.authors))
        self.assertEqual(self.aar.Title, str(self.title))
        self.assertEqual(self.aar.Citation, str(self.citation))
        self.assertEqual(self.aar.Comments, str(self.comments))
        self.assertEqual(self.aar.Correlating, self.correlating)
        self.assertEqual(self.aar.Data,self.data)

    def test_toSquareDistanceMatrix(self):
        """ AAIR1: Tests that _toSquareDistanceMatrix runs without returning an error """
        square = self.aar._toSquareDistanceMatrix()

    def test_toSquareDistanceMatrix_data_integrity_diagonal(self):
        """ AAIR1: Tests that diag = 0 when square matrix is built """
        square = self.aar._toSquareDistanceMatrix()
        # Test diagonal
        keys = 'ARNDCQEGHILKMFPSTWYV'
        for k in keys:
            self.assertEqual(square[k][k], 0.)

    def test_toSquareDistanceMatrix_data_integrity(self):
        """ AAIR1: Tests that _toSquareDistanceMatrix works right w/o stops """
        square = self.aar._toSquareDistanceMatrix()
        self.assertFloatEqualAbs(square['R']['A'], square['A']['R'])
        self.assertFloatEqualAbs(square['A']['R'], 1.)
        self.assertFloatEqualAbs(square['D']['N'], square['N']['D'])
        self.assertFloatEqualAbs(square['D']['N'], 1.)
        self.assertFloatEqualAbs(square['A']['C'], square['C']['A'])
        self.assertFloatEqualAbs(square['A']['C'], 4.)
        self.assertFloatEqualAbs(square['V']['A'], square['A']['V'])
        self.assertFloatEqualAbs(square['V']['A'], 19.)
        self.assertFloatEqualAbs(square['V']['Y'], square['Y']['V'])
        self.assertFloatEqualAbs(square['V']['Y'], 1.)

    def test_toSquareDistanceMatrix_data_integrity_w_stops(self):
        """ AAIR1: Tests that _toSquareDistanceMatrix works right w/ stops """
        square = self.aar._toSquareDistanceMatrix(include_stops=1)
        self.assertFloatEqualAbs(square['R']['A'], square['A']['R'])
        self.assertFloatEqualAbs(square['A']['R'], 1.)
        self.assertFloatEqualAbs(square['D']['N'], square['N']['D'])
        self.assertFloatEqualAbs(square['D']['N'], 1.)
        self.assertFloatEqualAbs(square['A']['C'], square['C']['A'])
        self.assertFloatEqualAbs(square['A']['C'], 4.)
        self.assertFloatEqualAbs(square['V']['A'], square['A']['V'])
        self.assertFloatEqualAbs(square['V']['A'], 19.)
        self.assertFloatEqualAbs(square['V']['Y'], square['Y']['V'])
        self.assertFloatEqualAbs(square['V']['Y'], 1.)
        self.assertFloatEqualAbs(square['V']['*'], None)
        self.assertFloatEqualAbs(square['*']['Y'], None)
        self.assertFloatEqualAbs(square['*']['*'], None)
        self.assertFloatEqualAbs(square['*']['R'], None)

    def test_toDistanceMatrix(self):
        """ AAIR1: Tests that toDistanceMatrix functions as expected """
        dm = self.aar.toDistanceMatrix()
        self.assertFloatEqualAbs(dm['R']['A'], dm['A']['R'])
        self.assertFloatEqualAbs(dm['A']['R'], 1.)
        self.assertFloatEqualAbs(dm['D']['N'], dm['N']['D'])
        self.assertFloatEqualAbs(dm['D']['N'], 1.)
        self.assertFloatEqualAbs(dm['A']['C'], dm['C']['A'])
        self.assertFloatEqualAbs(dm['A']['C'], 4.)
        self.assertFloatEqualAbs(dm['V']['A'], dm['A']['V'])
        self.assertFloatEqualAbs(dm['V']['A'], 19.)
        self.assertFloatEqualAbs(dm['V']['Y'], dm['Y']['V'])
        self.assertFloatEqualAbs(dm['V']['Y'], 1.)

    def test_toDistanceMatrix_w_stops(self):
        """ AAIR1: Tests that toDistanceMatrix works right w/ stops """
        square = self.aar.toDistanceMatrix(include_stops=1)
        self.assertFloatEqualAbs(square['R']['A'], square['A']['R'])
        self.assertFloatEqualAbs(square['A']['R'], 1.)
        self.assertFloatEqualAbs(square['D']['N'], square['N']['D'])
        self.assertFloatEqualAbs(square['D']['N'], 1.)
        self.assertFloatEqualAbs(square['A']['C'], square['C']['A'])
        self.assertFloatEqualAbs(square['A']['C'], 4.)
        self.assertFloatEqualAbs(square['V']['A'], square['A']['V'])
        self.assertFloatEqualAbs(square['V']['A'], 19.)
        self.assertFloatEqualAbs(square['V']['Y'], square['Y']['V'])
        self.assertFloatEqualAbs(square['V']['Y'], 1.)
        self.assertFloatEqualAbs(square['V']['*'], None)
        self.assertFloatEqualAbs(square['*']['Y'], None)
        self.assertFloatEqualAbs(square['*']['*'], None)
        self.assertFloatEqualAbs(square['*']['R'], None)

class AAIndex2RecordTests(AAIndexRecordTests):
    """ AAIR2: Tests AAIndex2Records class """

    def setUp(self):
        AAIndexRecordTests.setUp(self)

        # Build LTM data
        values = range(210)
        keys = 'ARNDCQEGHILKMFPSTWYV'

        self.LTMdata = dict.fromkeys(keys)

        i = 0
        for r in keys:
            new_row = dict.fromkeys(keys)
            for c in keys:
                if keys.find(c) <= keys.find(r):
                    new_row[c] = values[i]
                    i +=1
            self.LTMdata[r] = new_row



        self.aarLTM = AAIndex2Record(self.id, self.description,\
                self.LITDB_entry_num, self.authors, self.title,\
                self.citation, self.comments, self.LTMdata)

        # Build Square matrix data
        values = range(400)

        self.SQUdata = dict.fromkeys(keys)

        i = 0
        for r in keys:
            new_row = dict.fromkeys(keys)
            for c in keys:
                new_row[c] = values[i]
                i +=1
            self.SQUdata[r] = new_row

        self.aarSquare = AAIndex2Record(self.id, self.description,\
                self.LITDB_entry_num, self.authors, self.title,\
                self.citation, self.comments, self.SQUdata)

    def test_init(self):
        """ AAIR2: Tests init method returns with no errors"""

        test_aar = AAIndex2Record(self.id, self.description,\
                self.LITDB_entry_num, self.authors, self.title,\
                self.citation, self.comments, self.SQUdata)

    def test_init_data(self):
        """ AAIR2: Tests init correctly initializes data"""

        self.assertEqual(self.aarLTM.ID, str(self.id))
        self.assertEqual(self.aarLTM.Description, str(self.description))
        self.assertEqual(self.aarLTM.LITDBEntryNum,\
                str(self.LITDB_entry_num))
        self.assertEqual(self.aarLTM.Authors, str(self.authors))
        self.assertEqual(self.aarLTM.Title, str(self.title))
        self.assertEqual(self.aarLTM.Citation, str(self.citation))
        self.assertEqual(self.aarLTM.Comments, str(self.comments))

#    def test_matrix_values_col_by_row(self):
 #       """ Tests that keys and values correctly correspond in data LTM
#

#
 #           Also tests that reverse keys are same as forward keys.
#
 #       """
#
 #       data_matrix = self.aarLTM.Data
  #      self.assertEqual(data_matrix['A']['A'], 0)
  #      self.assertEqual(data_matrix['A']['R'], 1)
   #     self.assertEqual(data_matrix['R']['R'], 2)
    #    self.assertEqual(data_matrix['C']['H'], 40)
#        self.assertEqual(data_matrix['I']['M'], 87)
 #       self.assertEqual(data_matrix['D']['P'], 108)
 #       self.assertEqual(data_matrix['W']['V'], 207)
  #      self.assertEqual(data_matrix['Y']['V'], 208)
  #      self.assertEqual(data_matrix['V']['V'], 209)

 #   def test_LTM_values_row_by_col(self):
  #      """ Tests that keys are correctly linked to values in a LTM
#
 #           This tests that some random places hold the correct values.
  #          These are some randomly selected keys with hand calculated
   #         values.  Also included are the extreme values. Technically if
#            the first and last three are correct all values should be
 #           correct.
#
 #       """
  #      data_matrix = self.aarLTM.Data
   #     self.assertEqual(data_matrix['R']['A'], 1)
   #     self.assertEqual(data_matrix['H']['C'], 40)
   #     self.assertEqual(data_matrix['M']['I'], 87)
   #     self.assertEqual(data_matrix['P']['D'], 108)
   #     self.assertEqual(data_matrix['V']['W'], 207)
   #     self.assertEqual(data_matrix['V']['Y'], 208)
   #     self.assertEqual(data_matrix['A']['A'], 0)
    #    self.assertEqual(data_matrix['R']['R'], 2)
     #   self.assertEqual(data_matrix['V']['V'], 209)

    def test_Square_Matrix_values_row_by_col(self):
        """ AAIR2: Tests that key -> value pair integrity in Square matrix
        """
        data_matrix = self.aarSquare.Data
        self.assertEqual(data_matrix['R']['A'], 20)
        #self.assertEqual(data_matrix['H']['C'], 40)
        #self.assertEqual(data_matrix['M']['I'], 87)
        #self.assertEqual(data_matrix['P']['D'], 108)

        #self.assertEqual(data_matrix['V']['W'], 207)
        #self.assertEqual(data_matrix['V']['Y'], 208)
        self.assertEqual(data_matrix['A']['A'], 0)
        self.assertEqual(data_matrix['R']['R'], 21)
        self.assertEqual(data_matrix['V']['V'], 399)

    def test_toSquareDistanceMatrix_data_integrity(self):
        """ AAIR2: Tests that _toSquareDistanceMatrix works right w/o stops

        """
        square = self.aarSquare._toSquareDistanceMatrix()
        self.assertEqual(square['R']['A'], 20)
        self.assertEqual(square['A']['A'], 0)
        self.assertEqual(square['R']['R'], 21)
        self.assertEqual(square['V']['V'], 399)

    def test_toSquareDistanceMatrix_data_integrity_w_stops(self):
        """ AAIR2: Tests that _toSquareDistanceMatrix works right with stops
        """
        square = self.aarSquare._toSquareDistanceMatrix(include_stops=1)
        self.assertEqual(square['R']['A'], 20)
        self.assertEqual(square['A']['A'], 0)
        self.assertEqual(square['R']['R'], 21)
        self.assertEqual(square['V']['V'], 399)
        self.assertEqual(square['V']['*'], None)
        self.assertEqual(square['*']['Y'], None)
        self.assertEqual(square['*']['*'], None)
        self.assertEqual(square['*']['R'], None)

# Data for parser tests
        
fake_file_aaindex1 =\
"""
H ANDN920101
D alpha-CH chemical shifts (Andersen et al., 1992)
R LIT:1810048b PMID:1575719
A Andersen, N.H., Cao, B. and Chen, C.
T Peptide/protein structure analysis using the chemical shift index method:
  upfield alpha-CH values reveal dynamic helices and aL sites
J Biochem. and Biophys. Res. Comm. 184, 1008-1014 (1992)
C BUNA790102    0.949
I    A/L     R/K     N/M     D/F     C/P     Q/S     E/T     G/W     H/Y     I/V
    4.35    4.38    4.75    4.76    4.65    4.37    4.29    3.97    4.63    3.95
    4.17    4.36    4.52    4.66    4.44    4.50    4.35    4.70    4.60    3.95
//
H ARGP820101
D Hydrophobicity index (Argos et al., 1982)
R LIT:0901079b PMID:7151796
A Argos, P., Rao, J.K.M. and Hargrave, P.A.
T Structural prediction of membrane-bound proteins
J Eur. J. Biochem. 128, 565-575 (1982)
C JOND750101    1.000  SIMZ760101    0.967  GOLD730101    0.936
  TAKK010101    0.906  MEEJ810101    0.891  CIDH920105    0.867
  LEVM760106    0.865  CIDH920102    0.862  MEEJ800102    0.855
  MEEJ810102    0.853  CIDH920103    0.827  PLIV810101    0.820
  CIDH920104    0.819  LEVM760107    0.806  NOZY710101    0.800
  PARJ860101   -0.835  WOLS870101   -0.838  BULH740101   -0.854
I    A/L     R/K     N/M     D/F     C/P     Q/S     E/T     G/W     H/Y     I/V
    0.61    0.60    0.06    0.46    1.07      0.    0.47    0.07    0.61    2.22
    1.53    1.15    1.18    2.02    1.95    0.05    0.05    2.65    1.88    1.32
//
H TSAJ990102
D Volumes not including the crystallographic waters using the ProtOr (Tsai et
  al., 1999)
R PMID:10388571
A Tsai, J., Taylor, R., Chothia, C. and Gerstein, M.
T The packing density in proteins: standard radii and volumes
J J Mol Biol. 290, 253-266 (1999)
* (Cyh 113.7)
C TSAJ990101    1.000  CHOC750101    0.996  BIGC670101    0.992
  GOLD730102    0.991  KRIW790103    0.987  FAUJ880103    0.985
  GRAR740103    0.978  CHAM820101    0.978  CHOC760101    0.972
  FASG760101    0.940  LEVM760105    0.928  LEVM760102    0.918
  ROSG850101    0.909  DAWD720101    0.905  CHAM830106    0.896
  FAUJ880106    0.882  RADA880106    0.864  LEVM760107    0.861
  LEVM760106    0.841  RADA880103   -0.879
I    A/L     R/K     N/M     D/F     C/P     Q/S     E/T     G/W     H/Y     I/V
    90.0   194.0   124.7   117.3   103.3   149.4   142.2    64.9   160.0   163.9
   164.0   167.3   167.0   191.9   122.9    95.4   121.5   228.2   197.0   139.0
//
H JURD980101
D Modified Kyte-Doolittle hydrophobicity scale (Juretic et al., 1998)
R
A Juretic, D., Lucic, B., Zucic, D. and Trinajstic, N.
T Protein transmembrane structure: recognition and prediction by using
  hydrophobicity scales through preference functions
J Theoretical and Computational Chemistry, 5, 405-445 (1998)
C KYTJ820101    0.996  CHOC760103    0.967  NADH010102    0.931
  JANJ780102    0.928  NADH010101    0.925  EISD860103    0.901
  DESM900102    0.900  NADH010103    0.900  EISD840101    0.895
  RADA880101    0.893  MANP780101    0.887  WOLR810101    0.881
  PONP800103    0.879  JANJ790102    0.879  NADH010104    0.873
  CHOC760104    0.870  PONP800102    0.869  JANJ790101    0.868
  MEIH800103    0.861  PONP800101    0.858  NAKH920108    0.858
  RADA880108    0.857  PONP800108    0.856  ROSG850102    0.854
  PONP930101    0.849  RADA880107    0.842  BIOV880101    0.840
  MIYS850101    0.837  FAUJ830101    0.833  CIDH920104    0.832
  DESM900101    0.829  WARP780101    0.827  KANM800104    0.826
  LIFS790102    0.824  RADA880104    0.824  NADH010105    0.821
  NISK800101    0.816  NISK860101    0.808  BIOV880102    0.805
  ARGP820102    0.802  ARGP820103    0.800  VHEG790101   -0.814
  KRIW790101   -0.824  CHOC760102   -0.851  ROSM880101   -0.851
  MONM990101   -0.853  JANJ780103   -0.853  RACS770102   -0.855
  PRAM900101   -0.862  JANJ780101   -0.862  GUYH850101   -0.864
  GRAR740102   -0.864  MEIH800102   -0.879  KUHL950101   -0.884
  ROSM880102   -0.894  OOBM770101   -0.903
I    A/L     R/K     N/M     D/F     C/P     Q/S     E/T     G/W     H/Y     I/V
    1.10   -5.10   -3.50   -3.60    2.50   -3.68   -3.20   -0.64   -3.20    4.50
    3.80   -4.11    1.90    2.80   -1.90   -0.50   -0.70   -0.46    -1.3     4.2
//
H WILM950104
D Hydrophobicity coefficient in RP-HPLC, C18 with 0.1%TFA/2-PrOH/MeCN/H2O
  (Wilce et al. 1995)
R
A Wilce, M.C., Aguilar, M.I. and Hearn, M.T.
T Physicochemical basis of amino acid hydrophobicity scales: evaluation of four
  new scales of amino acid hydrophobicity coefficients derived from RP-HPLC of
  peptides
J Anal Chem. 67, 1210-1219 (1995)
C
I    A/L     R/K     N/M     D/F     C/P     Q/S     E/T     G/W     H/Y     I/V
   -2.34    1.60    2.81   -0.48    5.03    0.16    1.30   -1.06   -3.00    7.26
    1.09    1.56    0.62    2.57   -0.15    1.93    0.19    3.59   -2.58    2.06
//
H ARGP820103
D Membrane-buried preference parameters (Argos et al., 1982)
R LIT:0901079b PMID:7151796
A Argos, P., Rao, J.K.M. and Hargrave, P.A.
T Structural prediction of membrane-bound proteins
J Eur. J. Biochem. 128, 565-575 (1982)
C ARGP820102    0.961  MIYS850101    0.822  NAKH900106    0.810
  EISD860103    0.810  KYTJ820101    0.806  JURD980101    0.800
I    A/L     R/K     N/M     D/F     C/P     Q/S     E/T     G/W     H/Y     I/V
    1.56    0.45    0.27    0.14    1.23    0.51    0.23    0.62    0.29    1.67
    2.93    0.15    2.96    2.03    0.76    0.81    0.91    1.08    0.68    1.14
//
"""

fake_file_aaindex2 =\
"""
H ALTS910101
D The PAM-120 matrix (Altschul, 1991)
R LIT:1713145 PMID:2051488
A Altschul, S.F.
T Amino acid substitution matrices from an information theoretic perspective
J J. Mol. Biol. 219, 555-565 (1991)
M rows = ARNDCQEGHILKMFPSTWYV, cols = ARNDCQEGHILKMFPSTWYV
      3.
     -3.      6.
      0.     -1.      4.
      0.     -3.      2.      5.
     -3.     -4.     -5.     -7.      9.
     -1.      1.      0.      1.     -7.      6.
      0.     -3.      1.      3.     -7.      2.      5.
      1.     -4.      0.      0.     -5.     -3.     -1.      5.
     -3.      1.      2.      0.     -4.      3.     -1.     -4.      7.
     -1.     -2.     -2.     -3.     -3.     -3.     -3.     -4.     -4.      6.
     -3.     -4.     -4.     -5.     -7.     -2.     -4.     -5.     -3.      1.      5.
     -2.      2.      1.     -1.     -7.      0.     -1.     -3.     -2.     -2.     -4.      5.
     -2.     -1.     -3.     -4.     -6.     -1.     -4.     -4.     -4.      1.      3.      0.      8.
     -4.     -4.     -4.     -7.     -6.     -6.     -6.     -5.     -2.      0.      0.     -6.     -1.      8.
      1.     -1.     -2.     -2.     -3.      0.     -1.     -2.     -1.     -3.     -3.     -2.     -3.     -5.      6.
      1.     -1.      1.      0.     -1.     -2.     -1.      1.     -2.     -2.     -4.     -1.     -2.     -3.      1.      3.
      1.     -2.      0.     -1.     -3.     -2.     -2.     -1.     -3.      0.     -3.     -1.     -1.     -4.     -1.      2.      4.
     -7.      1.     -5.     -8.     -8.     -6.     -8.     -8.     -5.     -7.     -5.     -5.     -7.     -1.     -7.     -2.     -6.     12.
     -4.     -6.     -2.     -5.     -1.     -5.     -4.     -6.     -1.     -2.     -3.     -6.     -4.      4.     -6.     -3.     -3.     -1.      8.
      0.     -3.     -3.     -3.     -2.     -3.     -3.     -2.     -3.      3.      1.     -4.      1.     -3.     -2.     -2.      0.     -8.     -3.      5.
//
H BENS940103
D Log-odds scoring matrix collected in 74-100 PAM (Benner et al., 1994)
R LIT:2023094 PMID:7700864
A Benner, S.A., Cohen, M.A. and Gonnet, G.H.
T Amino acid substitution during functionally constrained divergent
  evolution of protein sequences
J Protein Engineering 7, 1323-1332 (1994)
* extrapolated to 250 PAM
M rows = ARNDCQEGHILKMFPSTWYV, cols = ARNDCQEGHILKMFPSTWYV
     2.4
    -0.8     4.8
    -0.2     0.3     3.6
    -0.3    -0.5     2.2     4.8
     0.3    -2.2    -1.8    -3.2    11.8
    -0.3     1.6     0.7     0.8    -2.6     3.0
    -0.1     0.3     1.0     2.9    -3.2     1.7     3.7
     0.6    -1.0     0.4     0.2    -2.0    -1.1    -0.5     6.6
    -1.0     1.0     1.2     0.4    -1.3     1.4     0.2    -1.6     6.1
    -0.8    -2.6    -2.8    -3.9    -1.2    -2.0    -2.9    -4.3    -2.3     4.0
    -1.4    -2.4    -3.1    -4.2    -1.6    -1.7    -3.1    -4.6    -1.9     2.8     4.2
    -0.4     2.9     0.9     0.4    -2.9     1.7     1.2    -1.1     0.6    -2.3    -2.4     3.4
    -0.8    -1.8    -2.2    -3.2    -1.2    -1.0    -2.2    -3.5    -1.5     2.6     2.9    -1.5     4.5
    -2.6    -3.5    -3.2    -4.7    -0.7    -2.8    -4.3    -5.4     0.0     0.9     2.1    -3.6     1.3     7.2
     0.4    -1.0    -1.0    -1.0    -3.1    -0.2    -0.7    -1.7    -1.0    -2.6    -2.2    -0.8    -2.4    -3.8     7.5
     1.1    -0.2     0.9     0.4     0.1     0.1     0.1     0.4    -0.3    -1.8    -2.2     0.0    -1.4    -2.6     0.5     2.1
     0.7    -0.3     0.4    -0.2    -0.6    -0.1    -0.2    -1.0    -0.5    -0.3    -1.1     0.1    -0.4    -2.2     0.1     1.4     2.5
    -4.1    -1.6    -4.0    -5.5    -0.9    -2.8    -4.7    -4.1    -1.0    -2.3    -0.9    -3.6    -1.3     3.0    -5.2    -3.4    -3.7    14.7
    -2.6    -2.0    -1.4    -2.8    -0.4    -1.8    -3.0    -4.3     2.5    -1.0    -0.1    -2.4    -0.5     5.3    -3.4    -1.9    -2.1     3.6     8.1
     0.1    -2.2    -2.2    -2.9    -0.2    -1.7    -2.1    -3.1    -2.1     3.2     1.9    -1.9     1.8     0.1    -1.9    -1.0     0.2    -2.9    -1.4     3.4
//
H QUIB020101
D STROMA score matrix for the alignment of known distant homologs
  (Qian-Goldstein, 2002)
R PMID:12211027
A Qian, B. and Goldstein, R.A.
T Optimization of a new score function for the generation of accurate
  alignments
J Proteins. 48, 605-610 (2002)
M rows = ARNDCQEGHILKMFPSTWYV, cols = ARNDCQEGHILKMFPSTWYV
     2.5
     0.2     5.2
     1.1     0.7     2.5
       1     0.1     3.3     5.3
     1.2    -1.3    -1.9    -3.1    11.5
    -0.1       2     1.9     1.1    -2.5     3.6
     1.2     1.9     2.3     3.2    -2.4     1.7     3.7
     1.4    -0.2     0.7     0.9    -1.3    -0.3     0.5     7.5
    -1.4     1.5     1.4     0.5    -1.7     1.4     0.3    -1.7     6.8
     0.3    -1.9    -2.4    -2.9    -3.2    -0.9    -3.1    -3.7    -1.8     4.5
    -0.2    -1.5    -2.4    -3.4    -1.6    -1.2    -1.5    -3.8    -2.4     3.4     5.2
    -0.2     3.4     1.6     1.4      -3     2.2     1.2     0.4     1.1    -1.5      -2     3.9
    -0.2    -1.4    -2.1    -2.8    -1.3    -0.6      -2    -3.8    -0.8     2.2     3.1    -0.5     5.4
    -1.6    -3.2    -2.5    -3.7    -0.8    -1.7   -13.7    -4.7    -0.9     2.2     3.7    -2.8     1.7       7
     0.7    -0.6    -0.1    -0.2    -3.6       1       0    -0.8    -2.1    -2.4    -1.4     0.2    -1.9    -4.1     8.1
     1.7     0.2     1.4     1.7     0.7     0.9     1.1     1.6    -0.1    -1.1    -0.8     1.4    -1.1    -2.5       2     2.8
     1.7     0.2     1.4     0.1     0.3    -0.1     1.6    -0.6    -0.2       0     0.3       1    -0.3    -0.8     1.1     2.6     0.4
    -3.3    -1.5      -4    -5.7    -0.5    -2.9    -4.7    -4.2    -1.2    -1.8    -1.2      -3    -0.6     3.7      -5    -2.8    -2.9    14.9
    -1.8    -0.9    -0.8    -2.9    -0.3    -1.5    -2.2    -4.8     2.9     0.2     0.8    -1.5     0.5     5.2    -3.3    -0.9    -0.8     4.9     8.1
     1.9    -2.8    -0.9    -2.5     0.7    -1.5    -1.3    -1.4    -2.5     4.5     3.4      -1     1.7     0.9    -1.1      -3     1.5    -2.5     0.3     4.2
//
H HENS920104
D BLOSUM50 substitution matrix (Henikoff-Henikoff, 1992)
R LIT:1902106 PMID:1438297
A Henikoff, S. and Henikoff, J.G.
T Amino acid substitution matrices from protein blocks
J Proc. Natl. Acad. Sci. USA 89, 10915-10919 (1992)
* #  Matrix made by matblas from blosum50.iij
* #  BLOSUM Clustered Scoring Matrix in 1/3 Bit Units
* #  Blocks Database = /data/blocks_5.0/blocks.dat
* #  Cluster Percentage: >= 50
* #  Entropy =   0.4808, Expected =  -0.3573
M rows = ARNDCQEGHILKMFPSTWYV, cols = ARNDCQEGHILKMFPSTWYV
       5      -2      -1      -2      -1      -1      -1       0      -2      -1      -2      -1      -1      -3      -1       1       0      -3      -2       0
      -2       7      -1      -2      -4       1       0      -3       0      -4      -3       3      -2      -3      -3      -1      -1      -3      -1      -3
      -1      -1       7       2      -2       0       0       0       1      -3      -4       0      -2      -4      -2       1       0      -4      -2      -3
      -2      -2       2       8      -4       0       2      -1      -1      -4      -4      -1      -4      -5      -1       0      -1      -5      -3      -4
      -1      -4      -2      -4      13      -3      -3      -3      -3      -2      -2      -3      -2      -2      -4      -1      -1      -5      -3      -1
      -1       1       0       0      -3       7       2      -2       1      -3      -2       2       0      -4      -1       0      -1      -1      -1      -3
      -1       0       0       2      -3       2       6      -3       0      -4      -3       1      -2      -3      -1      -1      -1      -3      -2      -3
       0      -3       0      -1      -3      -2      -3       8      -2      -4      -4      -2      -3      -4      -2       0      -2      -3      -3      -4
      -2       0       1      -1      -3       1       0      -2      10      -4      -3       0      -1      -1      -2      -1      -2      -3       2      -4
      -1      -4      -3      -4      -2      -3      -4      -4      -4       5       2      -3       2       0      -3      -3      -1      -3      -1       4
      -2      -3      -4      -4      -2      -2      -3      -4      -3       2       5      -3       3       1      -4      -3      -1      -2      -1       1
      -1       3       0      -1      -3       2       1      -2       0      -3      -3       6      -2      -4      -1       0      -1      -3      -2      -3
      -1      -2      -2      -4      -2       0      -2      -3      -1       2       3      -2       7       0      -3      -2      -1      -1       0       1
      -3      -3      -4      -5      -2      -4      -3      -4      -1       0       1      -4       0       8      -4      -3      -2       1       4      -1
      -1      -3      -2      -1      -4      -1      -1      -2      -2      -3      -4      -1      -3      -4      10      -1      -1      -4      -3      -3
       1      -1       1       0      -1       0      -1       0      -1      -3      -3       0      -2      -3      -1       5       2      -4      -2      -2
       0      -1       0      -1      -1      -1      -1      -2      -2      -1      -1      -1      -1      -2      -1       2       5      -3      -2       0
      -3      -3      -4      -5      -5      -1      -3      -3      -3      -3      -2      -3      -1       1      -4      -4      -3      15       2      -3
      -2      -1      -2      -3      -3      -1      -2      -3       2      -1      -1      -2       0       4      -3      -2      -2       2       8      -1
       0      -3      -3      -4      -1      -3      -3      -4      -4       4       1      -3       1      -1      -3      -2       0      -3      -1       5
//
H KOSJ950101
D Context-dependent optimal substitution matrices for exposed helix
  (Koshi-Goldstein, 1995)
R LIT:2124140 PMID:8577693
A Koshi, J.M. and Goldstein, R.A.
T Context-dependent optimal substitution matrices.
J Protein Engineering 8, 641-645 (1995)
M rows = -ARNDCQEGHILKMFPSTWYV, cols = -ARNDCQEGHILKMFPSTWYV
    55.7
     3.0     3.0
     3.0     3.0     0.4
     0.1     3.0     3.0     2.1
     3.0     3.0     3.0     0.1     1.9
     2.2     2.4     3.0     0.8     1.3     3.0
    25.6    47.2     1.5     1.0     0.7     0.3     1.9
     2.3     4.3     0.6     0.2     2.0     0.8     0.1     0.3
     3.1     2.8     3.7     0.4     0.1     2.0    14.8     0.9    62.7
     1.3     0.4     0.3     4.6     0.3     0.1     1.9     0.5     2.2     5.1
     0.6     0.2     0.4     1.9     1.5     0.4     0.2     0.3    15.2     0.2     0.5
    48.2     3.3     0.1     3.2     4.9     0.1     1.7     1.7     1.4     3.0     0.6     1.0
     0.1     9.7     2.7     0.7     1.1     1.5    15.9     3.9     1.4     7.3    52.1     0.3     0.9
    11.0     2.0     0.4     0.6     0.1     0.6     0.5     0.1     0.6     2.9     0.1     0.1     0.1     0.1
     9.4     1.5     0.1     1.5     1.6    73.6     0.1     2.6     0.1     0.1     2.1     4.0     0.1     0.1     0.8
     0.7     0.3     2.2     0.1     0.1     0.1     0.1     8.4     5.7     2.0     4.5     0.3    47.5     8.2     0.9     1.6
     0.1     3.4     7.8     0.5     0.1     0.7     5.3     2.2     0.2     0.7     0.5     5.2     5.3     1.0     1.5     8.6     0.1
     4.9    56.8     1.5     1.0     0.3     0.9     5.8     0.1     0.2     1.6     2.1     2.4     0.2     0.1     1.1    20.2     2.0     1.2
     2.3     3.3     0.1     0.4     0.1       6     4.8     0.8     0.1     0.1     1.4     0.3     0.6     0.1     1.2     0.6     0.1     0.5    13.3
     0.3     4.7     7.5     1.8     0.1     4.4     0.7     0.1    56.9     0.6     0.1     2.3     1.2     2.2     0.1     0.1     0.1     0.1     4.4     0.1
    18.4     0.1     0.1     0.1     0.1     0.1     0.4     0.1     0.1     0.1       5     2.6    10.8     1.2     3.5     1.3     0.1     0.1     3.4     0.1     0.1
//
H OVEJ920102
D Environment-specific amino acid substitution matrix for alpha residues
  (Overington et al., 1992)
R LIT:1811128 PMID:1304904
A Overington, J., Donnelly, D., Johnson, M.S., Sali, A. and Blundell, T.L.
T Environment-specific amino acid substitution tables: tertiary templates
  and prediction of protein folds
J Protein Science 1, 216-226 (1992)
M rows = ACDEFGHIKLMNPQRSTVWYJ-, cols = ACDEFGHIKLMNPQRSTVWYJ
   0.355   0.007   0.090   0.100   0.050   0.177   0.037   0.077   0.096   0.056   0.081   0.103   0.106   0.090   0.088   0.163   0.120   0.098   0.065   0.036   0.252
   0.001   0.901   0.000   0.000   0.000   0.000   0.000   0.004   0.001   0.000   0.000   0.003   0.000   0.006   0.006   0.004   0.002   0.000   0.007   0.000   0.000
   0.038   0.000   0.315   0.109   0.006   0.041   0.027   0.009   0.033   0.004   0.009   0.088   0.051   0.089   0.023   0.065   0.048   0.013   0.012   0.011   0.009
   0.044   0.011   0.111   0.305   0.011   0.048   0.026   0.011   0.059   0.013   0.009   0.068   0.069   0.086   0.053   0.033   0.045   0.017   0.012   0.018   0.000
   0.017   0.000   0.005   0.007   0.415   0.004   0.009   0.039   0.025   0.097   0.042   0.013   0.006   0.011   0.009   0.009   0.014   0.041   0.053   0.085   0.009
   0.065   0.000   0.070   0.042   0.006   0.370   0.017   0.022   0.029   0.013   0.015   0.036   0.043   0.031   0.013   0.068   0.049   0.014   0.009   0.021   0.045
   0.010   0.000   0.012   0.011   0.010   0.007   0.571   0.003   0.022   0.005   0.015   0.043   0.006   0.035   0.021   0.016   0.008   0.017   0.009   0.037   0.009
   0.029   0.014   0.009   0.008   0.048   0.021   0.004   0.325   0.017   0.076   0.107   0.018   0.007   0.007   0.015   0.014   0.033   0.112   0.016   0.030   0.018
   0.053   0.007   0.044   0.081   0.020   0.041   0.044   0.026   0.336   0.029   0.059   0.073   0.045   0.094   0.163   0.041   0.054   0.026   0.041   0.028   0.036
   0.038   0.000   0.006   0.018   0.210   0.019   0.004   0.139   0.033   0.415   0.225   0.033   0.016   0.041   0.028   0.029   0.026   0.133   0.037   0.057   0.036
   0.013   0.000   0.004   0.003   0.016   0.007   0.000   0.043   0.014   0.053   0.197   0.010   0.000   0.018   0.004   0.003   0.010   0.018   0.021   0.021   0.018
   0.031   0.007   0.057   0.035   0.010   0.026   0.054   0.012   0.034   0.012   0.013   0.195   0.015   0.066   0.026   0.037   0.046   0.012   0.002   0.048   0.000
   0.022   0.000   0.036   0.035   0.005   0.026   0.011   0.009   0.020   0.006   0.000   0.013   0.424   0.013   0.016   0.039   0.011   0.009   0.002   0.000   0.000
   0.025   0.011   0.045   0.039   0.011   0.021   0.031   0.004   0.045   0.015   0.035   0.059   0.015   0.183   0.029   0.030   0.030   0.008   0.007   0.025   0.009
   0.019   0.011   0.012   0.023   0.005   0.008   0.019   0.010   0.069   0.009   0.004   0.018   0.013   0.028   0.348   0.030   0.019   0.005   0.007   0.018   0.018
   0.086   0.021   0.075   0.047   0.012   0.079   0.033   0.020   0.041   0.020   0.009   0.089   0.082   0.069   0.063   0.264   0.096   0.028   0.005   0.020   0.054
   0.043   0.007   0.039   0.033   0.020   0.038   0.014   0.026   0.032   0.015   0.026   0.057   0.028   0.046   0.035   0.065   0.266   0.037   0.016   0.034   0.000
   0.055   0.000   0.018   0.021   0.069   0.022   0.044   0.178   0.025   0.111   0.016   0.018   0.025   0.017   0.015   0.129   0.060   0.350   0.012   0.043   0.162
   0.009   0.000   0.003   0.004   0.022   0.004   0.007   0.006   0.012   0.006   0.020   0.001   0.001   0.006   0.004   0.002   0.007   0.003   0.588   0.064   0.000
   0.009   0.000   0.006   0.006   0.046   0.006   0.029   0.014   0.007   0.013   0.031   0.033   0.003   0.020   0.010   0.007   0.017   0.016   0.078   0.377   0.027
   0.009   0.000   0.001   0.000   0.001   0.004   0.001   0.002   0.002   0.002   0.004   0.000   0.000   0.004   0.003   0.006   0.004   0.010   0.000   0.005   0.297
   0.028   0.004   0.041   0.074   0.010   0.029   0.017   0.022   0.050   0.031   0.033   0.031   0.045   0.039   0.028   0.047   0.034   0.032   0.002   0.021   0.000
//
"""

# Run tests if called from the command line
if __name__ == '__main__':
    main()


                                                                                                                          




#!/usr/bin/env python
"""Provides Filter, Organizer, GroupList.

Filter is a dictionary of {field_name:[filter_functions]}.
Organizer is initiated with a list of Filters and it is called on a list of
objects. It organizes all objects according to the given criteria (Filters).
GroupList stores a group of objects and a separte hierarchy (i.e. list
of elements or functions it matched).
"""

__author__ = "Sandra Smit"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Sandra Smit"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Sandra Smit"
__email__ = "sandra.smit@colorado.edu"
__status__ = "Development"

class Filter(dict):
    """Dictionary of functions, i.e. selection criteria"""

    def __init__(self,Name,functions_dict=None):
        """Returns a new Filter object with given name and funtions"""
        if not functions_dict:
            functions_dict = {}
        super(Filter, self).__init__(functions_dict)
        self.Name = Name

    def __call__(self, item):
        """Returns True if the item satisfies all criteria"""
        for field in self:
            for function in self[field]:
                try:
                    if field is None:
                        if not function(item):
                            return False
                    else:
                        if not function(getattr(item, field)):
                            return False
                except AttributeError:
                    return False
        return True

class GroupList(list):
    """GroupList is a list of objects accompanied by a hierarchy.

    The data can be of any type.
    Groups is a list that stores the hierarchy of things the data matched.
    E.g. filters, classifiers.

    Example GroupList: data = [1,2,3,4], groups = ['numbers','<5']
            GroupList: data = [5,6,7,8], groups = ['numbers','>=5']
    """

    def __init__(self, data, Groups=None):
        """Returns a new GroupList with data and Groups"""
        super(GroupList, self).__init__(data)
        self.Groups = Groups or []


class Organizer(list):
    """Organizer puts given objects into dict according to given criteria"""

    def _find_first_match(self, item):
        """Returns the name of the first criterion that applies to the item"""
        for criterion in self:
            if criterion(item):
                return criterion.Name

    def __call__(self, data, existing_results=None):
        """Puts the sequences into the dictionary according to the criteria"""
        temp_results = {None:[]}   #dict to store intermediate results
        results = []        #the final results
        
        try:
            groups = data.Groups
        except AttributeError:
            groups = []

        #figure out what criteria we have, and make keys for them
        for criterion in self:
            temp_results[criterion.Name] = []

        #partition each datum according to which filter it matches first
        for item in data:
            temp_results[self._find_first_match(item)].append(item)
        
        return [GroupList(value, groups + [key]) \
            for key, value in temp_results.items() if value]

def regroup(data):
    """Regroups data into matching categories.
    
    data is a list of GroupLists (a list with hierarchy info associated)
    e.g. regroup([GroupList([1,2],['a']),GroupList([8,9],['b']),
            GroupList([3,4],['a'])]
    will give: [[1,2,3,4],[8,9]] 
    """
    result = {}
    for group_list in data:
        analyses = tuple(group_list.Groups)
        
        try:
            result[analyses].extend(group_list)
        except:
            result[analyses] = []
            result[analyses].extend(group_list)
    return  [GroupList(result[k],list(k)) for k,v in result.items()]



#!/usr/bin/env python

import os
from cogent.format.pdb import PDBWriter, PDBXWriter
from cogent.format.xyzrn import XYZRNWriter
from cogent.parse.record import FileFormatError

__author__ = "Marcin Cieslik"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Marcin Cieslik"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Marcin Cieslik"
__email__ = "mpc4p@virginia.edu"
__status__ = "Development"

def save_to_filename(entities, filename, format, **kw):
    """Saves a  structure in a specified format into a given file name.
    Arguments:
        - entities: structure or entities to be written
        - filename: name of the structure file
        - format: structure file format
    """
    f = open(filename, 'w')
    try:
        write_to_file(f, entities, format, **kw)
    except Exception:
        try:
            os.unlink(filename)
        except Exception:
            pass
        raise
    f.close()

def write_to_file(f, entities, format, **kw):
    """Saves a structure in a specified format into a given file handle.
    Arguments:
        - entities: structure or entities to be written
        - filename: name of the structure file
        - format: structure file format
    """
    format = format.lower()
    if format not in WRITERS:
        raise FileFormatError("Unsupported file format %s" % format)
    writer = WRITERS[format]
    writer(f, entities, **kw)

# to add a new file format add it's suffix and class name here
WRITERS = {
        'pdb': PDBWriter,
        'pdbx': PDBXWriter,
        'xyzrn': XYZRNWriter
        }




#!/usr/bin/env python
# File created on 15 Jul 2011
from __future__ import division

__author__ = "AUTHOR_NAME"
__copyright__ = "COPYRIGHT_INFORMATION"
__credits__ = ["AUTHOR_NAME"]
__license__ = "GPL"
__version__ = "1.6.0dev"
__maintainer__ = "AUTHOR_NAME"
__email__ = "AUTHOR_EMAIL"
__status__ = "Development"



from cogent.util.option_parsing import parse_command_line_parameters, make_option

script_info = {}
script_info['brief_description'] = ""
script_info['script_description'] = ""
script_info['script_usage'] = [("","","")]
script_info['output_description']= ""
script_info['required_options'] = [\
 # Example required option
 #make_option('-i','--input_dir',type="existing_filepath",help='the input directory'),\
]
script_info['optional_options'] = [\
 # Example optional option
 #make_option('-o','--output_dir',type="new_dirpath",help='the output directory [default: %default]'),\
]
script_info['version'] = __version__



def main():
    option_parser, opts, args =\
       parse_command_line_parameters(**script_info)


if __name__ == "__main__":
    main()


#!/usr/bin/env python
#test_rdb.py
"""Unit test for RDB Parser
"""
from cogent.util.unit_test import TestCase, main
from cogent.parse.rdb import RdbParser, MinimalRdbParser,is_seq_label,\
    InfoMaker, create_acceptable_sequence
from cogent.core.sequence import Sequence, DnaSequence, RnaSequence
from cogent.core.info import Info
from cogent.parse.record import RecordError

__author__ = "Sandra Smit"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Sandra Smit", "Rob Knight"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Sandra Smit"
__email__ = "sandra.smit@colorado.edu"
__status__ = "Production"

class RdbTests(TestCase):
    """Tests for top-level functions in Rdb.py"""

    def test_is_seq_label(self):
        """is_seq_label should return True if a line starts with 'seq:'"""
        seq = 'seq:this is a sequence line'
        not_seq = 'this is not a sequence line'
        still_not_seq = 'this seq: is still not a sequence line'
        self.assertEqual(is_seq_label(seq),True)
        self.assertEqual(is_seq_label(not_seq),False)
        self.assertEqual(is_seq_label(still_not_seq),False)

    def test_create_acceptable_sequence(self):
        """create_acceptable_sequence: should handle 'o' and sec. struct"""
        f = create_acceptable_sequence
        # should keep any char accepted by RNA.Alphabet.DegenGapped
        s = "UCAG---NRYBDHKMNSRWVY?"
        self.assertEqual(f(s),s)
        # should replace 'o' by '?'
        s = "UCAG-oo-ACGU"
        self.assertEqual(f(s), "UCAG-??-ACGU")
        # should strip out secondary info
        s = "{UC^AG-[oo]-A(CG)U}"
        self.assertEqual(f(s), "UCAG-??-ACGU")
        # should leave other chars untouched
        s = "XYZ1234"
        self.assertEqual(f(s), "XYZ1234")

class InfoMakerTests(TestCase):
    """Tests for the Constructor InfoMaker. Should return an Info object"""
    
    def test_empty(self):
        """InfoMaker: should return empty Info from empty header"""
        empty_header = []
        obs = InfoMaker(empty_header)
        exp = Info()
        self.assertEqual(obs,exp)
    
    def test_full(self):
        """InfoMaker should return Info object with name, value pairs"""
        test_header = ['acc: X3402','abc:1','mty: ssu','seq: Mit. X3402',\
                        '','nonsense',':no_name']
        obs = InfoMaker(test_header)
        exp = Info()
        exp.rRNA = 'X3402'
        exp.abc = '1'
        exp.Species = 'Mit. X3402'
        exp.Gene = 'ssu'
        self.assertEqual(obs,exp)

class GenericRdbTest(TestCase):
    "SetUp data for all Rdb parsers"""
    
    def setUp(self):
        self.empty = []
        self.labels = 'mty:ssu\nseq:bac\n//\nttl:joe\nseq:mit\n//'.split('\n')
        self.nolabels = 'ACGUAGCUAGCUAC\nGCUGCAUCG\nAUCG\n//'.split('\n')
        self.oneseq = 'seq:H.Sapiens\nAGUCAUCUAGAUHCAUHC\n//'.split('\n')
        self.multiline = 'seq:H.Sapiens\nAGUCAUUAG\nAUHCAUHC\n//'.split('\n')
        self.threeseq =\
            'seq:bac\nAGU\n//\nseq:mit\nACU\n//\nseq:pla\nAAA\n//'.split('\n')
        self.twogood =\
            'seq:bac\n//\nseq:mit\nACU\n//\nseq:pla\nAAA\n//'.split('\n')
        self.oneX =\
            'seq:bac\nX\n//\nseq:mit\nACT\n//\nseq:pla\nAAA\n//'.split('\n')
        self.strange = 'seq:bac\nACGUXxAaKkoo---*\n//'.split('\n')

class MinimalRdbParserTests(GenericRdbTest):
    """Tests of MinimalRdbParser: returns (headerLines,sequence) tuples"""
    
    def test_empty(self):
        """MinimalRdbParser should return empty list from file w/o seqs"""
        self.assertEqual(list(MinimalRdbParser(self.empty)),[])
        self.assertEqual(list(MinimalRdbParser(self.nolabels, strict=False)),
                [])
        self.assertRaises(RecordError, list, MinimalRdbParser(self.nolabels))

    def test_only_labels(self):
        """MinimalRdbParser should return empty list from file w/o seqs"""
        #should fail if strict (the default)
        self.assertRaises(RecordError, list, 
            MinimalRdbParser(self.labels,strict=True))
        #if not strict, should skip the records
        self.assertEqual(list(MinimalRdbParser(self.labels, strict=False)), 
            [])

    def test_only_sequences(self):
        """MinimalRdbParser should return empty list form file w/o lables"""
        #should fail if strict (the default)
        self.assertRaises(RecordError, list,
            MinimalRdbParser(self.nolabels,strict=True))
        #if not strict, should skip the records
        self.assertEqual(list(MinimalRdbParser(self.nolabels, strict=False)), 
            [])

    def test_single(self):
        """MinimalRdbParser should read single record as (header,seq) tuple"""
        res = list(MinimalRdbParser(self.oneseq))
        self.assertEqual(len(res),1)
        first = res[0]
        self.assertEqual(first, (['seq:H.Sapiens'], 'AGUCAUCUAGAUHCAUHC'))
       
        res = list(MinimalRdbParser(self.multiline))
        self.assertEqual(len(res),1)
        first = res[0]
        self.assertEqual(first, (['seq:H.Sapiens'], 'AGUCAUUAGAUHCAUHC'))

    def test_multiple(self):
        """MinimalRdbParser should read multiple record correctly"""
        res = list(MinimalRdbParser(self.threeseq))
        self.assertEqual(len(res), 3)
        a, b, c = res
        self.assertEqual(a, (['seq:bac'], 'AGU'))
        self.assertEqual(b, (['seq:mit'], 'ACU'))
        self.assertEqual(c, (['seq:pla'], 'AAA'))

    def test_multiple_bad(self):
        """MinimalRdbParser should complain or skip bad records"""
        self.assertRaises(RecordError, list, MinimalRdbParser(self.twogood))
        f = list(MinimalRdbParser(self.twogood, strict=False))
        self.assertEqual(len(f), 2)
        a, b = f
        self.assertEqual(a, (['seq:mit'], 'ACU'))
        self.assertEqual(b, (['seq:pla'], 'AAA'))
        
    def test_strange(self):
        """MRP: handle strange char. according to constr. and strip off '*'"""
        f = list(MinimalRdbParser(self.strange))
        obs = f[0]
        exp = (['seq:bac'],'ACGUXxAaKkoo---')
        self.assertEqual(obs,exp)

class RdbParserTests(GenericRdbTest):
    """Tests for the RdbParser. Should return Sequence objects"""
    
    def test_empty(self):
        """RdbParser should return empty list from 'file' w/o labels"""
        self.assertEqual(list(RdbParser(self.empty)), [])
        self.assertEqual(list(RdbParser(self.nolabels, strict=False)),
            [])
        self.assertRaises(RecordError, list, RdbParser(self.nolabels))

    def test_only_labels(self):
        """RdbParser should return empty list from file w/o seqs"""
        #should fail if strict (the default)
        self.assertRaises(RecordError, list, 
            RdbParser(self.labels,strict=True))
        #if not strict, should skip the records
        self.assertEqual(list(RdbParser(self.labels, strict=False)), [])
        
    def test_only_sequences(self):
        """RdbParser should return empty list form file w/o lables"""
        #should fail if strict (the default)
        self.assertRaises(RecordError, list,
            RdbParser(self.nolabels,strict=True))
        #if not strict, should skip the records
        self.assertEqual(list(RdbParser(self.nolabels, strict=False)), 
            [])
    
    def test_single(self):
        """RdbParser should read single record as (header,seq) tuple"""
        res = list(RdbParser(self.oneseq))
        self.assertEqual(len(res),1)
        first = res[0]
        self.assertEqual(first, Sequence('AGUCAUCUAGAUHCAUHC'))
        self.assertEqual(first.Info, Info({'Species':'H.Sapiens',\
            'OriginalSeq':'AGUCAUCUAGAUHCAUHC'}))
       
        res = list(RdbParser(self.multiline))
        self.assertEqual(len(res),1)
        first = res[0]
        self.assertEqual(first, Sequence('AGUCAUUAGAUHCAUHC'))
        self.assertEqual(first.Info, Info({'Species':'H.Sapiens',\
            'OriginalSeq':'AGUCAUUAGAUHCAUHC'}))

    def test_single_constructor(self):
        """RdbParser should use constructors if supplied"""
        to_dna = lambda x, Info: DnaSequence(str(x).replace('U','T'), \
            Info=Info)
        f = list(RdbParser(self.oneseq, to_dna))
        self.assertEqual(len(f), 1)
        a = f[0]
        self.assertEqual(a, 'AGTCATCTAGATHCATHC')
        self.assertEqual(a.Info, Info({'Species':'H.Sapiens',\
            'OriginalSeq':'AGUCAUCUAGAUHCAUHC'}))

        def alternativeConstr(header_lines):
            info = Info()
            for line in header_lines:
                all = line.strip().split(':',1)
                #strip out empty lines, lines without name, lines without colon
                if not all[0] or len(all) != 2: 
                    continue
                name = all[0].upper()
                value = all[1].strip().upper()
                info[name] = value
            return info
        
        f = list(RdbParser(self.oneseq, to_dna, alternativeConstr))
        self.assertEqual(len(f), 1)
        a = f[0]
        self.assertEqual(a, 'AGTCATCTAGATHCATHC')
        exp_info = Info({'OriginalSeq':'AGUCAUCUAGAUHCAUHC',\
            'Refs':{}, 'SEQ':'H.SAPIENS'})
        self.assertEqual(a.Info, Info({'OriginalSeq':'AGUCAUCUAGAUHCAUHC',\
            'Refs':{}, 'SEQ':'H.SAPIENS'}))

    def test_multiple_constructor_bad(self):
        """RdbParser should complain or skip bad records w/ constructor"""

        def dnastrict(x, **kwargs):
            try:
                return DnaSequence(x,  **kwargs)
            except Exception:
                raise RecordError, "Could not convert sequence"
       
        self.assertRaises(RecordError, list, RdbParser(self.oneX,dnastrict))
        f = list(RdbParser(self.oneX, dnastrict, strict=False))
        self.assertEqual(len(f), 2)
        a, b = f

        self.assertEqual(a, 'ACT')
        self.assertEqual(a.Info,Info({'Species':'mit','OriginalSeq':'ACT'}))
        self.assertEqual(b, 'AAA')
        self.assertEqual(b.Info,Info({'Species':'pla','OriginalSeq':'AAA'}))

    def test_full(self):
        """RdbParser: full data, valid and invalid"""
        # when only good record, should work independent of strict
        r1 = RnaSequence("-??GG-UGAA--CGCU---ACGU-N???---",\
            Info=Info({'Species': "unidentified Thermus OPB AF027020",\
            'Refs':{'rRNA':['AF027020']},\
            'OriginalSeq':'-o[oGG-U{G}AA--C^GC]U---ACGU-Nooo---'}))
        r2 = RnaSequence("---CGAUCG--UAUACG-N???-",\
            Info=Info({'Species':'Thermus silvanus X84211',\
            'Refs':{'rRNA':['X84211']},\
            'OriginalSeq':'---CGAU[C(G){--UA}U]ACG-Nooo-'}))
        obs = list(RdbParser(RDB_LINES_ONLY_GOOD.split('\n'), strict=True))
        self.assertEqual(len(obs), 2)
        self.assertEqual(obs[0], r1)
        self.assertEqual(str(obs[0]), str(r1))
        self.assertEqual(obs[0].Info, r1.Info)
        self.assertEqual(obs[1], r2)
        self.assertEqual(str(obs[1]), str(r2))
        self.assertEqual(obs[1].Info, r2.Info)

        obs = list(RdbParser(RDB_LINES_ONLY_GOOD.split('\n'), strict=False))
        self.assertEqual(len(obs), 2)
        self.assertEqual(obs[0], r1)
        self.assertEqual(str(obs[0]), str(r1))
        self.assertEqual(obs[0].Info, r1.Info)

        # when strict, should raise error on invalid record
        f = RdbParser(RDB_LINES_GOOD_BAD.split('\n'), strict=True)
        self.assertRaises(RecordError, list, f)
        # when not strict, malicious record is skipped
        obs = list(RdbParser(RDB_LINES_GOOD_BAD.split('\n'), strict=False))
        self.assertEqual(len(obs), 2)
        self.assertEqual(obs[0], r1)
        self.assertEqual(str(obs[0]), str(r1))
        self.assertEqual(obs[0].Info, r1.Info)
        self.assertEqual(obs[1], r2)
        self.assertEqual(str(obs[1]), str(r2))
        self.assertEqual(obs[1].Info, r2.Info)

RDB_LINES_ONLY_GOOD=\
"""acc:AF027020
seq: unidentified Thermus OPB AF027020
-o[oGG-U{G}AA--C^GC]U---ACGU-Nooo---
//
acc:X84211
seq: Thermus silvanus X84211
---CGAU[C(G){--UA}U]ACG-Nooo-
//
"""

RDB_LINES_GOOD_BAD=\
"""acc:AF027020
seq: unidentified Thermus OPB AF027020
-o[oGG-U{G}AA--C^GC]U---ACGU-Nooo---
//
acc:ABC123
seq: E. coli
---ACGU-Nooo-RYXQ-
//
acc:X84211
seq: Thermus silvanus X84211
---CGAU[C(G){--UA}U]ACG-Nooo-
//
"""

if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Unit tests for the bowtie default output parser.
   Compatible with bowtie version 0.12.5
"""

from cogent.parse.bowtie import BowtieOutputParser, BowtieToTable
from cogent.util.unit_test import TestCase, main
from cogent import LoadTable

__author__ = "Gavin Huttley, Anuj Pahwa"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Rob Knight","Peter Maxwell", "Gavin Huttley", "Anuj Pahwa"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Gavin Huttley"
__email__ = "gavin.huttley@anu.edu.au"
__status__ = "Development"

fname = 'data/bowtie_output.map'
expected = [['GAPC_0015:6:1:1283:11957#0/1', '-', 'Mus', 66047927, 'TGTATATATAAACATATATGGAAACTGAATATATATACATTATGTATGTATATATGTATATGTTATATATACATA', 'IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII', 0, ['55:A>G', '64:C>A']],
        ['GAPC_0015:6:1:1394:18813#0/1', '+', 'Mus', 77785518, 'ATGAAATTCCTAGCCAAATGGATGGACCTGGAGGGCATCATC', 'IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII', 447, []],
        ['GAPC_0015:6:1:1560:18056#0/1', '+', 'Mus', 178806665, 'TAGATAAAGGCTCTGTTTTTCATCATTGAGAAATTGTTATTTTTCTGATGTTATA', 'IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII', 0, ['9:T>G']],
        ['GAPC_0015:6:1:1565:19849#0/1', '+', 'Mus', 116516430, 'ACCATTTGCTTGGAAAATTGTTTTCCAGCCTTTCACTCTGAG', 'IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII', 141, []],
        ['GAPC_0015:6:1:1591:17397#0/1', '-', 'Mus', 120440696, 'TCTAAATCTGTTCATTAATTAAGCCTGTTTCCATGTCCTTGGTCTTAAGACCAATCTGTTATGCGGGTGTGA', 'IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII', 0, ['70:A>C', '71:G>T']]]

class BowtieOutputTest(TestCase):
    
    def test_parsing(self):
        """make sure that the bowtie output file is parsed properly"""
        parser = BowtieOutputParser(fname)
        header = parser.next()
        index = 0
        for row in parser:
            self.assertEqual(row, expected[index])
            index += 1
    
    def test_psl_to_table(self):
        """make sure that the table is built without any errors"""
        table = BowtieToTable(fname)
    
    def test_getting_seq_coords(self):
        """get correct information from the table"""
        table = BowtieToTable(fname)
        index = 0
        for row in table:
            query_name = row['Query Name']
            strand_direction = row['Strand Direction']
            query_offset = row['Offset']
            self.assertEqual(query_name, expected[index][0])
            self.assertEqual(strand_direction, expected[index][1])
            self.assertEqual(query_offset, expected[index][3])
            index += 1
        
    def test_no_row_converter(self):
        """setting row_converter=None returns strings"""
        # straight parser
        parser = BowtieOutputParser(fname, row_converter=None)
        header = parser.next()
        for index, row in enumerate(parser):
            query_offset = row[3]
            other_matches = row[6]
            self.assertEqual(query_offset, str(expected[index][3]))
            self.assertEqual(other_matches, str(expected[index][6]))
        
        # table
        table = BowtieToTable(fname, row_converter=None)
        for index, row in enumerate(table):
            query_offset = row['Offset']
            other_matches = row['Other Matches']
            self.assertEqual(query_offset, str(expected[index][3]))
            self.assertEqual(other_matches, str(expected[index][6]))
        

if __name__ == "__main__":
    main()



#!/usr/bin/env python

from cogent.util.unit_test import TestCase, main
from cogent.core.tree import PhyloNode
from numpy import array
import numpy
Float = numpy.core.numerictypes.sctype2char(float)
from cogent.cluster.UPGMA import find_smallest_index, condense_matrix, \
        condense_node_order, UPGMA_cluster, inputs_from_dict2D, upgma
from cogent.util.dict2d import Dict2D

__author__ = "Rob Knight"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Peter Maxwell", "Rob Knight"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Production"

class UPGMATests(TestCase):
    """test the functions to cluster using UPGMA using numpy"""

    def setUp(self):
        """creates inputs"""
        self.pairwise_distances = {('a', 'b'): 1.0,
        ('a', 'c'):4.0,
        ('a', 'd'):20.0,
        ('a', 'e'):22.0,
        ('b', 'c'):5.0,
        ('b', 'd'):21.0,
        ('b', 'e'):23.0,
        ('c', 'd'):10.0,
        ('c', 'e'):12.0,
        ('d', 'e'):2.0}
        #create a list of PhyloNode objects
        a, b, c, d, e = map(PhyloNode, 'abcde')
        self.node_order = [a, b, c, d, e]
        #create a numpy matrix object to cluster
        self.matrix = array(([9999999, 1, 4, 20, 22], \
                        [1, 9999999, 5, 21, 23], \
                        [4, 5, 9999999, 10, 12], \
                        [20, 21, 10, 9999999, 2], \
                        [22, 23, 12, 2, 9999999]), Float)
        #create a numpy matrix with zero diagonals to test diagonal mask 
        self.matrix_zeros = array(([0, 1, 4, 20, 22], \
                        [1, 0, 5, 21, 23], \
                        [4, 5, 0, 10, 12], \
                        [20, 21, 10, 0, 2], \
                        [22, 23, 12, 2, 0]), Float)
        
        #create a numpy matrix with zero diagonals to test diagonal mask 
        self.matrix_five = array(([5, 1, 4, 20, 22], \
                        [1, 5, 5, 21, 23], \
                        [4, 5, 5, 10, 12], \
                        [20, 21, 10, 5, 2], \
                        [22, 23, 12, 2, 5]), Float)
    
    def test_UPGMA_cluster(self):
        """upgma works on pairwise distance dict
        """
        pairwise_dist = self.pairwise_distances
        cluster = upgma(pairwise_dist)
        self.assertEqual(str(cluster), '(((b:0.5,a:0.5)edge.1:1.75,c:2.25)edge.0:5.875,(d:1.0,e:1.0)edge.2:7.125)root;')
        
    def test_find_smallest_index(self):
        """find_smallest_index returns the index of smallest value in array
        """
        matrix = self.matrix
        index = find_smallest_index(matrix)
        self.assertEqual(index, (0,1))

    def test_condense_matrix(self):
        """condense_array joins two rows and columns identified by indices
        """
        matrix = self.matrix
        index = find_smallest_index(matrix)
        result = condense_matrix(matrix, index, 9999999999)
        self.assertFloatEqual(result[0, 0], 5000000.0)
        self.assertEqual(result[1, 4], 9999999999)
        self.assertEqual(result[0, 1], 9999999999)
        self.assertEqual(result[0, 2], 4.5)
        self.assertEqual(result[2, 0], 4.5)
        self.assertEqual(result[0, 4], 22.5)
        self.assertEqual(result[4, 4], 9999999)
        self.assertEqual(result[4, 0], 22.5)

    def test_condense_node_order(self):
        """condense_node_order condenses nodes in list based on index info
        """
        matrix = self.matrix
        index = find_smallest_index(matrix)
        node_order = self.node_order
        node_order = condense_node_order(matrix, index, node_order)
        self.assertEqual(node_order[1], None)
        self.assertEqual(node_order[0].__str__(), '(a:0.5,b:0.5);')
        self.assertEqual(node_order[2].__str__(), 'c;')
        self.assertEqual(node_order[3].__str__(), 'd;')
        self.assertEqual(node_order[4].__str__(), 'e;')

    def test_upgma_cluster(self):
        """UPGMA_cluster clusters nodes based on info in a matrix with UPGMA
        """
        matrix = self.matrix
        node_order = self.node_order
        large_number = 9999999999
        tree = UPGMA_cluster(matrix, node_order, large_number)
        self.assertEqual(str(tree), \
                '(((a:0.5,b:0.5):1.75,c:2.25):5.875,(d:1.0,e:1.0):7.125);')
    
    def test_UPGMA_cluster_diag(self):
        """UPGMA_cluster works when the diagonal has lowest values
        """
        #test that checking the diagonal works
        matrix = self.matrix_zeros
        node_order = self.node_order
        large_number = 9999999999
        tree = UPGMA_cluster(matrix, node_order, large_number)
        self.assertEqual(str(tree), \
                '(((a:0.5,b:0.5):1.75,c:2.25):5.875,(d:1.0,e:1.0):7.125);')
    
    def test_UPGMA_cluster_diag(self):
        """UPGMA_cluster works when the diagonal has intermediate values
        """
        #test that checking the diagonal works
        matrix = self.matrix_five
        node_order = self.node_order
        large_number = 9999999999
        tree = UPGMA_cluster(matrix, node_order, large_number)
        self.assertEqual(str(tree), \
                '(((a:0.5,b:0.5):1.75,c:2.25):5.875,(d:1.0,e:1.0):7.125);')

    def test_inputs_from_dict2D(self):
        """inputs_from_dict2D makes an array object and PhyloNode list"""
        matrix = [('1', '2', 0.86), ('2', '1', 0.86), \
                ('1', '3', 0.92), ('3', '1', 0.92), ('2', '3', 0.67), \
                ('3', '2', 0.67)]
        row_order = ['3', '2', '1']
        matrix_d2d = Dict2D(matrix, RowOrder=row_order, \
                ColOrder=row_order, Pad=True, Default = 999999999999999)
        matrix_array, PhyloNode_order = inputs_from_dict2D(matrix_d2d)
        self.assertFloatEqual(matrix_array[0][2], 0.92)
        self.assertFloatEqual(matrix_array[1][0], 0.67)
        self.assertEqual(PhyloNode_order[0].Name, '3')
        self.assertEqual(PhyloNode_order[2].Name, '1')

#run if called from command line
if __name__ == '__main__':
       main()



#!/usr/bin/env python
"""Contains classes that represent biological sequence data. These
provide generic biological sequence manipulation functions, plus functions
that are critical for the EVOLVE calculations.

WARNING: Do not import sequence classes directly! It is expected that you will
access them through the moltype module. Sequence classes depend on information
from the MolType that is _only_ available after MolType has been imported.

Sequences are intended to be immutable. This is not enforced by the code for
performance reasons, but don't alter the MolType or the sequence data after
creation.
"""
from __future__ import division
from annotation import Map, Feature, _Annotatable
from cogent.util.transform import keep_chars, for_seq, per_shortest, \
    per_longest
from cogent.util.misc import DistanceFromMatrix
from cogent.core.genetic_code import DEFAULT as DEFAULT_GENETIC_CODE, \
    GeneticCodes
from cogent.parse import gff
from cogent.format.fasta import fasta_from_sequences
from cogent.core.info import Info as InfoClass
from numpy import array, zeros, put, nonzero, take, ravel, compress, \
    logical_or, logical_not, arange
from numpy.random import permutation
from operator import eq, ne
from random import shuffle
import re
import warnings

__author__ = "Rob Knight, Gavin Huttley, and Peter Maxwell"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Rob Knight", "Peter Maxwell", "Gavin Huttley",
                    "Matthew Wakefield", "Daniel McDonald"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Production"

ARRAY_TYPE = type(array(1))

#standard distance functions: left  because generally useful
frac_same = for_seq(f=eq, aggregator=sum, normalizer=per_shortest)
frac_diff = for_seq(f=ne, aggregator=sum, normalizer=per_shortest)

class SequenceI(object):
    """Abstract class containing Sequence interface.
    
    Specifies methods that Sequence delegates to its MolType, and methods for
    detecting gaps.
    """
    #String methods delegated to self._seq -- remember to override if self._seq
    #isn't a string in your base class, but it's probably better to make
    #self._seq a property that contains the string.
    LineWrap = None #used for formatting FASTA strings
    def __str__(self):
        """__str__ returns self._seq unmodified."""
        return self._seq
    
    def toFasta(self, make_seqlabel=None):
        """Return string of self in FASTA format, no trailing newline
        
        Arguments:
            - make_seqlabel: callback function that takes the seq object and
              returns a label str
        """
        return fasta_from_sequences([self], make_seqlabel = make_seqlabel,
                        line_wrap=self.LineWrap)
    
    def translate(self, *args, **kwargs):
        """translate() delegates to self._seq."""
        return self._seq.translate(*args, **kwargs)
    
    def count(self, item):
        """count() delegates to self._seq."""
        return self._seq.count(item)
    
    def __cmp__(self, other):
        """__cmp__ compares based on the sequence string."""
        return cmp(self._seq, other)
    
    def __hash__(self):
        """__hash__ behaves like the sequence string for dict lookup."""
        return hash(self._seq)
    
    def __contains__(self, other):
        """__contains__ checks whether other is in the sequence string."""
        return other in self._seq
    
    def shuffle(self):
        """returns a randomized copy of the Sequence object"""
        randomized_copy_list = list(self)
        shuffle(randomized_copy_list)
        return self.__class__(''.join(randomized_copy_list), Info=self.Info)
    
    def complement(self):
        """Returns complement of self, using data from MolType.
        
        Always tries to return same type as item: if item looks like a dict,
        will return list of keys.
        """
        return self.__class__(self.MolType.complement(self), Info=self.Info)
    
    def stripDegenerate(self):
        """Removes degenerate bases by stripping them out of the sequence."""
        return self.__class__(self.MolType.stripDegenerate(self), Info=self.Info)
    
    def stripBad(self):
        """Removes any symbols not in the alphabet."""
        return self.__class__(self.MolType.stripBad(self), Info=self.Info)
    
    def stripBadAndGaps(self):
        """Removes any symbols not in the alphabet, and any gaps."""
        return self.__class__(self.MolType.stripBadAndGaps(self), Info=self.Info)
    
    def rc(self):
        """Returns reverse complement of self w/ data from MolType.
        
        Always returns same type self.
        """
        return self.__class__(self.MolType.rc(self), Info=self.Info)
    
    def isGapped(self):
        """Returns True if sequence contains gaps."""
        return self.MolType.isGapped(self)
    
    def isGap(self, char=None):
        """Returns True if char is a gap.
        
        If char is not supplied, tests whether self is gaps only.
        """
        if char is None:    #no char - so test if self is all gaps
            return len(self) == self.countGaps()
        else:
            return self.MolType.isGap(char)
    
    def isDegenerate(self):
        """Returns True if sequence contains degenerate characters."""
        return self.MolType.isDegenerate(self)
    
    def isValid(self):
        """Returns True if sequence contains no items absent from alphabet."""
        return self.MolType.isValid(self)
    
    def isStrict(self):
        """Returns True if sequence contains only monomers."""
        return self.MolType.isStrict(self)
    
    def firstGap(self):
        """Returns the index of the first gap in the sequence, or None."""
        return self.MolType.firstGap(self)
    
    def firstDegenerate(self):
        """Returns the index of first degenerate symbol in sequence, or None."""
        return self.MolType.firstDegenerate(self)
    
    def firstInvalid(self):
        """Returns the index of first invalid symbol in sequence, or None."""
        return self.MolType.firstInvalid(self)
    
    def firstNonStrict(self):
        """Returns the index of first non-strict symbol in sequence, or None."""
        return self.MolType.firstNonStrict(self)
    
    def disambiguate(self, method='strip'):
        """Returns a non-degenerate sequence from a degenerate one.
        
        method can be 'strip' (deletes any characters not in monomers or gaps)
        or 'random'(assigns the possibilities at random, using equal
        frequencies).
        """
        return self.__class__(self.MolType.disambiguate(self, method), \
            Info=self.Info)
    
    def degap(self):
        """Deletes all gap characters from sequence."""
        return self.__class__(self.MolType.degap(self), Info=self.Info)
    
    def gapList(self):
        """Returns list of indices of all gaps in the sequence, or []."""
        return self.MolType.gapList(self)
    
    def gapVector(self):
        """Returns vector of True or False according to which pos are gaps."""
        return self.MolType.gapVector(self)
    
    def gapMaps(self):
        """Returns dicts mapping between gapped and ungapped positions."""
        return self.MolType.gapMaps(self)
    
    def countGaps(self):
        """Counts the gaps in the specified sequence."""
        return self.MolType.countGaps(self)
    
    def countDegenerate(self):
        """Counts the degenerate bases in the specified sequence."""
        return self.MolType.countDegenerate(self)
    
    def possibilities(self):
        """Counts number of possible sequences matching the sequence.
        
        Uses self.Degenerates to decide how many possibilites there are at
        each position in the sequence.
        """
        return self.MolType.possibilities(self)
    
    def MW(self, method='random', delta=None):
        """Returns the molecular weight of (one strand of) the sequence.
        
        If the sequence is ambiguous, uses method (random or strip) to
        disambiguate the sequence.
        
        If delta is passed in, adds delta per strand (default is None, which
        uses the alphabet default. Typically, this adds 18 Da for terminal
        water. However, note that the default nucleic acid weight assumes
        5' monophosphate and 3' OH: pass in delta=18.0 if you want 5' OH as
        well.
        
        Note that this method only calculates the MW of the coding strand. If
        you want the MW of the reverse strand, add self.rc().MW(). DO NOT
        just multiply the MW by 2: the results may not be accurate due to
        strand bias, e.g. in mitochondrial genomes.
        """
        return self.MolType.MW(self, method, delta)
    
    def canMatch(self, other):
        """Returns True if every pos in self could match same pos in other.
        
        Truncates at length of shorter sequence.
        Gaps are only allowed to match other gaps.
        """
        return self.MolType.canMatch(self, other)
    
    def canMismatch(self, other):
        """Returns True if any position in self could mismatch with other.
        
        Truncates at length of shorter sequence.
        Gaps are always counted as matches.
        """
        return self.MolType.canMismatch(self, other)
    
    def mustMatch(self, other):
        """Returns True if all positions in self must match positions in other."""
        return self.MolType.mustMatch(self, other)
    
    def canPair(self, other):
        """Returns True if self and other could pair.
        
        Pairing occurs in reverse order, i.e. last position of other with
        first position of self, etc.
        
        Truncates at length of shorter sequence.
        Gaps are only allowed to pair with other gaps, and are counted as 'weak'
        (same category as GU and degenerate pairs).
        
        NOTE: second must be able to be reverse
        """
        return self.MolType.canPair(self, other)
    
    def canMispair(self, other):
        """Returns True if any position in self could mispair with other.
        
        Pairing occurs in reverse order, i.e. last position of other with
        first position of self, etc.
        
        Truncates at length of shorter sequence.
        Gaps are always counted as possible mispairs, as are weak pairs like GU.
        """
        return self.MolType.canMispair(self, other)
    
    def mustPair(self, other):
        """Returns True if all positions in self must pair with other.
        
        Pairing occurs in reverse order, i.e. last position of other with
        first position of self, etc.
        """
        return not self.MolType.canMispair(self, other)
    
    def diff(self, other):
        """Returns number of differences between self and other.
        
        NOTE: truncates at the length of the shorter sequence. Case-sensitive.
        """
        return self.distance(other)
    
    def distance(self, other, function=None):
        """Returns distance between self and other using function(i,j).
        
        other must be a sequence.
        
        function should be a function that takes two items and returns a
        number. To turn a 2D matrix into a function, use
        cogent.util.miscs.DistanceFromMatrix(matrix).
        
        NOTE: Truncates at the length of the shorter sequence.
        
        Note that the function acts on two _elements_ of the sequences, not
        the two sequences themselves (i.e. the behavior will be the same for
        every position in the sequences, such as identity scoring or a function
        derived from a distance matrix as suggested above). One limitation of
        this approach is that the distance function cannot use properties of
        the sequences themselves: for example, it cannot use the lengths of the
        sequences to normalize the scores as percent similarities or percent
        differences.
        
        If you want functions that act on the two sequences themselves, there
        is no particular advantage in making these functions methods of the
        first sequences by passing them in as parameters like the function
        in this method. It makes more sense to use them as standalone functions.
        The factory function cogent.util.transform.for_seq is useful for
        converting per-element functions into per-sequence functions, since it
        takes as parameters a per-element scoring function, a score aggregation
        function, and a normalization function (which itself takes the two
        sequences as parameters), returning a single function that combines
        these functions and that acts on two complete sequences.
        """
        if function is None:
            #use identity scoring function
            function = lambda a, b : a != b
        distance = 0
        for first, second in zip(self, other):
            distance += function(first, second)
        return distance
    
    def matrixDistance(self, other, matrix):
        """Returns distance between self and other using a score matrix.
        
        WARNING: the matrix must explicitly contain scores for the case where
        a position is the same in self and other (e.g. for a distance matrix,
        an identity between U and U might have a score of 0). The reason the
        scores for the 'diagonals' need to be passed explicitly is that for
        some kinds of distance matrices, e.g. log-odds matrices, the 'diagonal'
        scores differ from each other. If these elements are missing, this
        function will raise a KeyError at the first position that the two
        sequences are identical.
        """
        return self.distance(other, DistanceFromMatrix(matrix))
    
    def fracSame(self, other):
        """Returns fraction of positions where self and other are the same.
        
        Truncates at length of shorter sequence.
        Note that fracSame and fracDiff are both 0 if one sequence is empty.
        """
        return frac_same(self, other)
    
    def fracDiff(self, other):
        """Returns fraction of positions where self and other differ.
        
        Truncates at length of shorter sequence.
        Note that fracSame and fracDiff are both 0 if one sequence is empty.
        """
        return frac_diff(self, other)
    
    def fracSameGaps(self, other):
        """Returns fraction of positions where self and other share gap states.
        
        In other words, if self and other are both all gaps, or both all
        non-gaps, or both have gaps in the same places, fracSameGaps will
        return 1.0. If self is all gaps and other has no gaps, fracSameGaps
        will return 0.0. Returns 0 if one sequence is empty.
        
        Uses self's gap characters for both sequences.
        """
        if not self or not other:
            return 0.0
        
        is_gap = self.MolType.Gaps.__contains__
        return sum([is_gap(i) == is_gap(j) for i,j in zip(self, other)]) \
            /min(len(self),len(other))
    
    def fracDiffGaps(self, other):
        """Returns frac. of positions where self and other's gap states differ.
        
        In other words, if self and other are both all gaps, or both all
        non-gaps, or both have gaps in the same places, fracDiffGaps will
        return 0.0. If self is all gaps and other has no gaps, fracDiffGaps
        will return 1.0.
        
        Returns 0 if one sequence is empty.
        
        Uses self's gap characters for both sequences.
        """
        if not self or not other:
            return 0.0
        return 1.0 - self.fracSameGaps(other)
    
    def fracSameNonGaps(self, other):
        """Returns fraction of non-gap positions where self matches other.
        
        Doesn't count any position where self or other has a gap.
        Truncates at the length of the shorter sequence.
        
        Returns 0 if one sequence is empty.
        """
        if not self or not other:
            return 0.0
        
        is_gap = self.MolType.Gaps.__contains__
        count = 0
        identities = 0
        for i, j in zip(self, other):
            if is_gap(i) or is_gap(j):
                continue
            count += 1
            if i == j:
                identities += 1
        
        if count:
            return identities/count
        else:   #there were no positions that weren't gaps
            return 0
    
    def fracDiffNonGaps(self, other):
        """Returns fraction of non-gap positions where self differs from other.
        
        Doesn't count any position where self or other has a gap.
        Truncates at the length of the shorter sequence.
        
        Returns 0 if one sequence is empty. Note that this means that
        fracDiffNonGaps is _not_ the same as 1 - fracSameNonGaps, since both
        return 0 if one sequence is empty.
        """
        if not self or not other:
            return 0.0
        
        is_gap = self.MolType.Gaps.__contains__
        count = 0
        diffs = 0
        for i, j in zip(self, other):
            if is_gap(i) or is_gap(j):
                continue
            count += 1
            if i != j:
                diffs += 1
        
        if count:
            return diffs/count
        else:   #there were no positions that weren't gaps
            return 0
    
    def fracSimilar(self, other, similar_pairs):
        """Returns fraction of positions where self[i] is similar to other[i].
        
        similar_pairs must be a dict such that d[(i,j)] exists if i and j are
        to be counted as similar. Use PairsFromGroups in cogent.util.misc to
        construct such a dict from a list of lists of similar residues.
        
        Truncates at the length of the shorter sequence.
        
        Note: current implementation re-creates the distance function each
        time, so may be expensive compared to creating the distance function
        using for_seq separately.
        
        Returns 0 if one sequence is empty.
        """
        if not self or not other:
            return 0.0
        
        return for_seq(f = lambda x, y: (x,y) in similar_pairs, \
            normalizer=per_shortest)(self, other)

    def withTerminiUnknown(self):
        """Returns copy of sequence with terminal gaps remapped as missing."""
        gaps = self.gapVector()
        first_nongap = last_nongap = None
        for i, state in enumerate(gaps):
            if not state:
                if first_nongap is None:
                    first_nongap = i
                last_nongap = i
        missing = self.MolType.Missing
        if first_nongap is None:    #sequence was all gaps
            result = self.__class__([missing for i in len(self)],Info=self.Info)
        else:
            prefix = missing*first_nongap
            mid = str(self[first_nongap:last_nongap+1])
            suffix = missing*(len(self)-last_nongap-1)
            result = self.__class__(prefix + mid + suffix, Info=self.Info)
        return result

class Sequence(_Annotatable, SequenceI):
    """Holds the standard Sequence object. Immutable."""
    MolType = None      #connected to ACSII when moltype is imported
    
    def __init__(self, Seq='',Name=None, Info=None, check=True, \
            preserve_case=False, gaps_allowed=True, wildcards_allowed=True):
        """Initialize a sequence.
        
        Arguments:
            Seq: the raw sequence string, default is ''
            
            Name: the sequence name
            
            check: if True (the default), validates against the MolType
        """
        if Name is None and hasattr(Seq, 'Name'):
            Name = Seq.Name
        self.Name = Name
        orig_seq = Seq
        if isinstance(Seq, Sequence):
            Seq = Seq._seq
        elif isinstance(Seq, ModelSequence):
            Seq = str(Seq)
        elif type(Seq) is not str:
            try:
                Seq = ''.join(Seq)
            except TypeError:
                Seq = ''.join(map(str, Seq))
        Seq = self._seq_filter(Seq)
        if not preserve_case and not Seq.isupper():
            Seq = Seq.upper()
        self._seq = Seq
        
        if check:
            self.MolType.verifySequence(self._seq, gaps_allowed, \
                wildcards_allowed)
        
        if not isinstance(Info, InfoClass):
            try:
                Info = InfoClass(Info)
            except TypeError:
                Info = InfoClass()
        if hasattr(orig_seq, 'Info'):
            try:
                Info.update(orig_seq.Info)
            except:
                pass
        self.Info = Info
        
        if isinstance(orig_seq, _Annotatable):
            self.copyAnnotations(orig_seq)
    
    def _seq_filter(self, seq):
        """Returns filtered seq; used to do DNA/RNA conversions."""
        return seq
    
    def getColourScheme(self, colours):
        return {}
        #dict([(motif,colours.black) for motif in self.MolType])
    
    def getColorScheme(self, colors):   #alias to support US spelling
        return self.getColourScheme(colours=colors)
    
    def copyAnnotations(self, other):
        self.annotations = other.annotations[:]
    
    def annotateFromGff(self, f):
        first_seqname = None
        for (seqname, source, feature, start, end, score, strand,
                frame, attributes, comments) in gff.GffParser(f):
            if first_seqname is None:
                first_seqname = seqname
            else:
                assert seqname == first_seqname, (seqname, first_seqname)
            feat_label = gff.parse_attributes(attributes)
            self.addFeature(feature, feat_label, [(start, end)])
    
    def withMaskedAnnotations(self, annot_types, mask_char=None, shadow=False):
        """returns a sequence with annot_types regions replaced by mask_char
        if shadow is False, otherwise all other regions are masked.
        
        Arguments:
            - annot_types: annotation type(s)
            - mask_char: must be a character valid for the seq MolType. The
              default value is the most ambiguous character, eg. '?' for DNA
            - shadow: whether to mask the annotated regions, or everything but
              the annotated regions"""
        if mask_char is None:
            ambigs = [(len(v), c) for c,v in self.MolType.Ambiguities.items()]
            ambigs.sort()
            mask_char = ambigs[-1][1]
        assert mask_char in self.MolType, 'Invalid mask_char %s' % mask_char
        
        annotations = []
        annot_types = [annot_types, [annot_types]][isinstance(annot_types, str)]
        for annot_type in annot_types:
            annotations += self.getAnnotationsMatching(annot_type)
        
        region = self.getRegionCoveringAll(annotations)
        if shadow:
            region = region.getShadow()
        
        i = 0
        segments = []
        for b, e in region.getCoordinates():
            segments.append(self._seq[i:b])
            segments.append(mask_char * (e-b))
            i = e
        segments.append(self._seq[i:])
        
        new = self.__class__(''.join(segments), Name=self.Name, check=False,
                            Info=self.Info)
        new.annotations = self.annotations[:]
        return new
    
    def gappedByMapSegmentIter(self, map, allow_gaps=True, recode_gaps=False):
        for span in map.spans:
            if span.lost:
                if allow_gaps:
                    unknown = span.terminal or recode_gaps
                    seg = "-?"[unknown] * span.length
                else:
                    raise ValueError('Gap(s) in map %s' % map)
            else:
                seg = self._seq[span.Start:span.End]
                if span.Reverse:
                    complement = self.MolType.complement
                    seg = [complement(base) for base in seg[::-1]]
                    seg = ''.join(seg)
            yield seg
    
    def gappedByMapMotifIter(self, map):
        for segment in self.gappedByMapSegmentIter(map):
            for motif in segment:
                yield motif
    
    def gappedByMap(self, map, recode_gaps=False):
        segments = self.gappedByMapSegmentIter(map, True, recode_gaps)
        new = self.__class__(''.join(segments),
            Name=self.Name, check=False, Info=self.Info)
        annots = self._slicedAnnotations(new, map)
        new.annotations = annots
        return new
    
    def _mapped(self, map):
        # Called by generic __getitem__
        segments = self.gappedByMapSegmentIter(map, allow_gaps=False)
        new = self.__class__(''.join(segments), self.Name, Info=self.Info)
        return new
    
    def __add__(self, other):
        """Adds two sequences (other can be a string as well)."""
        if hasattr(other, 'MolType'):
            if self.MolType != other.MolType:
                raise ValueError, "MolTypes don't match: (%s,%s)" % \
                    (self.MolType, other.MolType)
            other_seq = other._seq
        else:
            other_seq = other
        new_seq = self.__class__(self._seq + other_seq)
        # Annotations which extend past the right end of the left sequence
        # or past the left end of the right sequence are dropped because
        # otherwise they will annotate the wrong part of the constructed
        # sequence.
        left = [a for a in self._shiftedAnnotations(new_seq, 0)
                if a.map.End <= len(self)]
        if hasattr(other, '_shiftedAnnotations'):
            right = [a for a in other._shiftedAnnotations(new_seq, len(self))
                if a.map.Start >= len(self)]
            new_seq.annotations = left + right
        else:
            new_seq.annotations = left
        return new_seq
    
    def __repr__(self):
        myclass = '%s' % self.__class__.__name__
        myclass = myclass.split('.')[-1]
        if len(self) > 10:
            seq = str(self._seq[:7]) + '... %s' % len(self)
        else:
            seq = str(self._seq)
        return "%s(%s)" % (myclass, seq)
    
    def getTracks(self, policy):
        return policy.tracksForSequence(self)
    
    def getName(self):
        """Return the sequence name -- should just use Name instead."""
        
        return self.Name
    
    def __len__(self):
        return len(self._seq)
    
    def __iter__(self):
        return iter(self._seq)
    
    def gettype(self):
        """Return the sequence type."""
        
        return self.MolType.label
    
    def resolveambiguities(self):
        """Returns a list of tuples of strings."""
        ambigs = self.MolType.resolveAmbiguity
        return [ambigs(motif) for motif in self._seq]
    
    def slidingWindows(self, window, step, start=None, end=None):
        """Generator function that yield new sequence objects
        of a given length at a given interval.
        Arguments:
            - window: The length of the returned sequence
            - step: The interval between the start of the returned
              sequence objects
            - start: first window start position
            - end: last window start position
        """
        start = [start, 0][start is None]
        end = [end, len(self)-window+1][end is None]
        end = min(len(self)-window+1, end)
        if start < end and len(self)-end >= window-1:
            for pos in xrange(start, end, step):
                yield self[pos:pos+window]
    
    def getInMotifSize(self, motif_length=1, log_warnings=True):
        """returns sequence as list of non-overlapping motifs
        
        Arguments:
            - motif_length: length of the motifs
            - log_warnings: whether to notify of an incomplete terminal motif"""
        seq = self._seq
        if motif_length == 1:
            return seq
        else:
            length = len(seq)
            remainder = length % motif_length
            if remainder and log_warnings:
                warnings.warn('Dropped remainder "%s" from end of sequence' %
                        seq[-remainder:])
            return [seq[i:i+motif_length]
                    for i in range(0, length-remainder, motif_length)]
    
    def parseOutGaps(self):
        gapless = []
        segments = []
        nongap = re.compile('([^%s]+)' % re.escape("-"))
        for match in nongap.finditer(self._seq):
            segments.append(match.span())
            gapless.append(match.group())
        map = Map(segments, parent_length=len(self)).inverse()
        seq = self.__class__(
            ''.join(gapless),
            Name = self.getName(), Info=self.Info)
        if self.annotations:
            seq.annotations = [a.remappedTo(seq, map) for a in self.annotations]
        return (map, seq)
    

class ProteinSequence(Sequence):
    """Holds the standard Protein sequence. MolType set in moltype module."""
    pass

class ProteinWithStopSequence(Sequence):
    """Holds the standard Protein sequence, allows for stop codon

    MolType set in moltype module
    """
    pass

class NucleicAcidSequence(Sequence):
    """Base class for DNA and RNA sequences. Abstract."""
    PROTEIN = None #will set in moltype
    CodonAlphabet = None   #will set in moltype
    def reversecomplement(self):
        """Converts a nucleic acid sequence to its reverse complement.
        Synonymn for rc."""
        return self.rc()
    
    def rc(self):
        """Converts a nucleic acid sequence to its reverse complement."""
        complement = self.MolType.rc(self)
        rc = self.__class__(complement, Name=self.Name, Info=self.Info)
        self._annotations_nucleic_reversed_on(rc)
        return rc
    
    def _gc_from_arg(self, gc):
        # codon_alphabet is being deprecated in favor of genetic codes.
        if gc is None:
            gc = DEFAULT_GENETIC_CODE
        elif isinstance(gc, (int, basestring)):
            gc = GeneticCodes[gc]
        return gc
    
    def hasTerminalStop(self, gc=None, allow_partial=False):
        """Return True if the sequence has a terminal stop codon.
        
        Arguments:
            - gc: genetic code object
            - allow_partial: if True and the sequence length is not dividisble
              by 3, ignores the 3' terminal incomplete codon
        """
        gc = self._gc_from_arg(gc)
        codons = self._seq
        divisible_by_3 = len(codons) % 3 == 0
        
        if not allow_partial and not divisible_by_3:
            raise ValueError("seq length not divisible by 3")
        
        if not divisible_by_3:
            return False
        
        return codons and gc.isStop(codons[-3:])
    
    def withoutTerminalStopCodon(self, gc=None, allow_partial=False):
        """Removes a terminal stop codon from the sequence
        
        Arguments:
            - gc: genetic code object
            - allow_partial: if True and the sequence length is not divisible
              by 3, ignores the 3' terminal incomplete codon
        """
        gc = self._gc_from_arg(gc)
        codons = self._seq
        divisible_by_3 = len(codons) % 3 == 0
        
        if not allow_partial and not divisible_by_3:
            raise ValueError("seq length not divisible by 3")
        
        if divisible_by_3 and codons and gc.isStop(codons[-3:]):
            codons = codons[:-3]
        
        return self.__class__(codons, Name=self.Name, Info=self.Info)
    
    def getTranslation(self, gc=None):
        gc = self._gc_from_arg(gc)
        codon_alphabet = self.CodonAlphabet(gc).withGapMotif()
        # translate the codons
        translation = []
        for posn in range(0, len(self._seq)-2, 3):
            orig_codon = self._seq[posn:posn+3]
            resolved = codon_alphabet.resolveAmbiguity(orig_codon)
            trans = []
            for codon in resolved:
                if codon == '---':
                    aa = '-'
                else:
                    assert '-' not in codon
                    aa = gc[codon]
                    if aa == '*':
                        continue
                trans.append(aa)
            if not trans:
                raise ValueError(orig_codon)
            aa = self.PROTEIN.whatAmbiguity(trans)
            translation.append(aa)
        
        translation = self.PROTEIN.makeSequence(
                Seq=''.join(translation), Name=self.Name)
        
        return translation
    
    def getOrfPositions(self, gc=None, atg=False):
        gc = self._gc_from_arg(gc)
        orfs = []
        start = None
        protein = self.getTranslation(gc=gc)
        for (posn, aa) in enumerate(protein):
            posn *= 3
            if aa == '*':
                if start is not None:
                    orfs.append((start,posn))
                start = None
            else:
                if start is None:
                    if (not atg) or gc.isStart(self[posn:posn+3]):
                        start = posn
        if start is not None:
            orfs.append((start, posn+3))
        return orfs
    
    def toRna(self):
        """Returns copy of self as RNA."""
        return RnaSequence(self)
    
    def toDna(self):
        """Returns copy of self as DNA."""
        return DnaSequence(self)
    

class DnaSequence(NucleicAcidSequence):
    
    def getColourScheme(self, colours):
        return {
                'A': colours.black,
                'T': colours.red,
                'C': colours.blue,
                'G': colours.green,
                }
    
    def _seq_filter(self, seq):
        """Converts U to T."""
        return seq.replace('u','t').replace('U','T')
    

class RnaSequence(NucleicAcidSequence):
    
    def getColourScheme(self, colours):
        return {
                'A': colours.black,
                'U': colours.red,
                'C': colours.blue,
                'G': colours.green,
                }
    
    def _seq_filter(self, seq):
        """Converts T to U."""
        return seq.replace('t','u').replace('T','U')
    

class ABSequence(Sequence):
    """Used for two-state modeling; MolType set in moltypes."""
    pass

class ByteSequence(Sequence):
    """Used for storing arbitrary bytes."""
    def __init__(self, Seq='', Name=None, Info=None, check=False, \
            preserve_case=True):
        return super(ByteSequence, self).__init__(Seq, Name=Name, Info=Info, \
                check=check, preserve_case=preserve_case)
    

class ModelSequenceBase(object):
    """Holds the information for a non-degenerate sequence. Mutable.
    
    A ModelSequence is an array of indices of symbols, where those symbols are
    defined by an Alphabet. This representation of Sequence is convenient for
    counting symbol frequencies or tuple frequencies, remapping data (e.g. for
    reverse-complement), looking up model parameters, etc. Its main drawback is
    that the sequences can no longer be treated as strings, and conversion
    to/from strings can be fairly time-consuming. Also, any symbol not in the
    Alphabet cannot be represented at all.
    
    A sequence can have a Name, which will be used for output in formats
    such as FASTA.
    
    A sequence Class has an alphabet (which can be overridden in instances
    where necessary), a delimiter used for string conversions, a LineWrap
    for wrapping characters into lines for e.g. FASTA output.
    
    Note that a ModelSequence _must_ have an Alphabet, not a MolType,
    because it is often important to store just a subset of the possible
    characters (e.g. the non-degenerate bases) for modeling purposes.
    """
    Alphabet = None     #REPLACE IN SUBCLASSES
    MolType = None      #REPLACE IN SUBCLASSES
    Delimiter = ''      #Used for string conversions
    LineWrap = 80       #Wrap sequences at 80 characters by default.
    
    def __init__(self, data='', Alphabet=None, Name=None, Info=None, \
        check='ignored'):
        """Initializes sequence from data and alphabet.
        
        WARNING: Does not validate the data or alphabet for compatibility.
        This is for speed. Use isValid() to check whether the data
        is consistent with the alphabet.
        
        WARNING: If data has name and/or Info, gets ref to same object rather
        than copying in each case.
        """
        if Name is None and hasattr(data, 'Name'):
            Name = data.Name
        if Info is None and hasattr(data, 'Info'):
            Info = data.Info
        #set the label
        self.Name = Name
        #override the class alphabet if supplied
        if Alphabet is not None:
            self.Alphabet = Alphabet
        #if we haven't already set self._data (e.g. in a subclass __init__),
        #guess the data type and set it here
        if not hasattr(self, '_data'):
            #if data is a sequence, copy its data and alphabet
            if isinstance(data, ModelSequence):
                self._data = data._data
                self.Alphabet = data.Alphabet
            #if it's an array
            elif type(data) == ARRAY_TYPE:
                self._data = data
            else:    #may be set in subclass init
                self._from_sequence(data)
        
        self.MolType = self.Alphabet.MolType
        self.Info = Info

    def __getitem__(self, *args):
        """__getitem__ returns char or slice, as same class."""
        if len(args) == 1 and not isinstance(args[0], slice):
            result = array([self._data[args[0]]])
        else:
            result = self._data.__getitem__(*args)
        return self.__class__(result)
    
    def __cmp__(self, other):
        """__cmp__ compares based on string"""
        return cmp(str(self), other)
    
    def _from_sequence(self, data):
        """Fills self using the values in data, via the Alphabet."""
        if self.Alphabet:
            self._data = array(self.Alphabet.toIndices(data), \
                self.Alphabet.ArrayType)
        else:
            self._data = array(data)
    
    def __str__(self):
        """Uses alphabet to convert self to string, using delimiter."""
        if hasattr(self.Alphabet, 'toString'):
            return self.Alphabet.toString(self._data)
        else:
            return self.Delimiter.join(map(str, \
                self.Alphabet.fromIndices(self._data)))
    
    def __len__(self):
        """Returns length of data."""
        return len(self._data)
    
    def toFasta(self, make_seqlabel=None):
        """Return string of self in FASTA format, no trailing newline
        
        Arguments:
            - make_seqlabel: callback function that takes the seq object and
              returns a label str
        """
        return fasta_from_sequences([self], make_seqlabel = make_seqlabel,
                        line_wrap=self.LineWrap)
    
    def toPhylip(self, name_len=28, label_len=30):
        """Return string of self in one line for PHYLIP, no newline.
        
        Default: max name length is 28, label length is 30.
        """
        return str(self.Name)[:name_len].ljust(label_len) + str(self)
    
    def isValid(self):
        """Checks that no items in self are out of the Alphabet range."""
        return self._data == self._data.clip(m, 0, len(self.Alphabet)-1)
    
    def toKwords(self, k, overlapping=True):
        """Turns sequence into sequence of its k-words.
        
        Just returns array, not Sequence object."""
        alpha_len = len(self.Alphabet)
        seq = self._data
        seq_len = len(seq)
        if overlapping:
            num_words = seq_len - k + 1
        else:
            num_words, remainder = divmod(seq_len, k)
            last_index = num_words * k
        result = zeros(num_words)
        for i in range(k):
            if overlapping:
                curr_slice = seq[i:i+num_words]
            else:
                curr_slice = seq[i:last_index+i:k]
            result *= alpha_len
            result += curr_slice
        return result
    
    def __iter__(self):
        """iter returns characters of self, rather than slices."""
        if hasattr(self.Alphabet, 'toString'):
            return iter(self.Alphabet.toString(self._data))
        else:
            return iter(self.Alpabet.fromIndices(self._data))
    
    def tostring(self):
        """tostring delegates to self._data."""
        return self._data.tostring()
    
    def gaps(self):
        """Returns array containing 1 where self has gaps, 0 elsewhere.
        
        WARNING: Only checks for standard gap character (for speed), and
        does not check for ambiguous gaps, etc.
        """
        return self._data == self.Alphabet.GapIndex
    
    def nongaps(self):
        """Returns array contining 0 where self has gaps, 1 elsewhere.
        
        WARNING: Only checks for standard gap character (for speed), and
        does not check for ambiguous gaps, etc.
        """
        return self._data != self.Alphabet.GapIndex
    
    def regap(self, other, strip_existing_gaps=False):
        """Inserts elements of self into gaps specified by other.

        WARNING: Only checks for standard gap character (for speed), and
        does not check for ambiguous gaps, etc.
        """
        if strip_existing_gaps:
            s = self.degap()
        else:
            s = self
        c = self.__class__
        a = self.Alphabet.Gapped
        result = zeros(len(other),a.ArrayType)+a.GapIndex
        put(result, nonzero(other.nongaps()), s._data)
        return c(result)
    
    def degap(self):
        """Returns ungapped copy of self, not changing alphabet."""
        if not hasattr(self.Alphabet, 'Gap') or self.Alphabet.Gap is None:
            return self.copy()
        d = take(self._data, nonzero(logical_not(self.gapArray()))[0])
        return self.__class__(d, Alphabet=self.Alphabet, Name=self.Name, \
            Info=self.Info)
    
    def copy(self):
        """Returns copy of self, always separate object."""
        return self.__class__(self._data.copy(), Alphabet=self.Alphabet, \
            Name=self.Name, Info=self.Info)

    def __contains__(self, item):
        """Returns true if item in self (converts to strings)."""
        return item in str(self)

    def disambiguate(self, *args, **kwargs):
        """Disambiguates self using strings/moltype. Should recode if demand."""
        return self.__class__(self.MolType.disambiguate(str(self), \
            *args,**kwargs))

    def distance(self, other, function=None, use_indices=False):
        """Returns distance between self and other using function(i,j).
        
        other must be a sequence.
        
        function should be a function that takes two items and returns a
        number. To turn a 2D matrix into a function, use
        cogent.util.miscs.DistanceFromMatrix(matrix).

        use_indices: if False, maps the indices onto items (e.g. assumes 
        function relates the characters). If True, uses the indices directly.
        
        NOTE: Truncates at the length of the shorter sequence.
        
        Note that the function acts on two _elements_ of the sequences, not
        the two sequences themselves (i.e. the behavior will be the same for
        every position in the sequences, such as identity scoring or a function
        derived from a distance matrix as suggested above). One limitation of
        this approach is that the distance function cannot use properties of
        the sequences themselves: for example, it cannot use the lengths of the
        sequences to normalize the scores as percent similarities or percent
        differences.
        
        If you want functions that act on the two sequences themselves, there
        is no particular advantage in making these functions methods of the
        first sequences by passing them in as parameters like the function
        in this method. It makes more sense to use them as standalone functions.
        The factory function cogent.util.transform.for_seq is useful for
        converting per-element functions into per-sequence functions, since it
        takes as parameters a per-element scoring function, a score aggregation
        function, and a normalization function (which itself takes the two
        sequences as parameters), returning a single function that combines
        these functions and that acts on two complete sequences.
        """
        if function is None:
            #use identity scoring
            shortest = min(len(self), len(other))
            if not hasattr(other, '_data'):
                other = self.__class__(other)
            distance = (self._data[:shortest] != other._data[:shortest]).sum()
        else:
            distance = 0
            if use_indices:
                self_seq = self._data
                if hasattr(other, '_data'):
                    other_seq = other._data
            else:
                self_seq = self.Alphabet.fromIndices(self._data)
                if hasattr(other, '_data'):
                    other_seq = other.Alphabet.fromIndices(other._data)
                else:
                    other_seq = other
            for first, second in zip(self_seq, other_seq):
                distance += function(first, second)
        return distance

    def matrixDistance(self, other, matrix, use_indices=False):
        """Returns distance between self and other using a score matrix.
        
        if use_indices is True (default is False), assumes that matrix is
        an array using the same indices that self uses.

        WARNING: the matrix must explicitly contain scores for the case where
        a position is the same in self and other (e.g. for a distance matrix,
        an identity between U and U might have a score of 0). The reason the
        scores for the 'diagonals' need to be passed explicitly is that for
        some kinds of distance matrices, e.g. log-odds matrices, the 'diagonal'
        scores differ from each other. If these elements are missing, this
        function will raise a KeyError at the first position that the two
        sequences are identical.
        """
        return self.distance(other, DistanceFromMatrix(matrix))

    def shuffle(self):
        """Returns shuffled copy of self"""
        return self.__class__(permutation(self._data), Info=self.Info)
    
    def gapArray(self):
        """Returns array of 0/1 indicating whether each position is a gap."""
        gap_indices = []
        a = self.Alphabet
        for c in self.MolType.Gaps:
            if c in a:
                gap_indices.append(a.index(c))
        gap_vector = None
        for i in gap_indices:
            if gap_vector is None:
                gap_vector = self._data == i
            else:
                gap_vector = logical_or(gap_vector, self._data == i)
        return gap_vector

    def gapIndices(self):
        """Returns array of indices of gapped positions in self."""
        return self.gapArray().nonzero()[0]

    def fracSameGaps(self, other):
        """Returns fraction of positions where gaps match other's gaps.
        """
        if not other:
            return 0
        self_gaps = self.gapArray()
        if hasattr(other, 'gapArray'):
            other_gaps = other.gapArray()
        elif hasattr(other, 'gapVector'):
            other_gaps = array(other.gapVector())
        else:
            other_gaps = array(self.MolType.gapVector(other))
        min_len = min(len(self), len(other))
        self_gaps, other_gaps = self_gaps[:min_len], other_gaps[:min_len]
        return (self_gaps == other_gaps).sum()/float(min_len)
 

class ModelSequence(ModelSequenceBase, SequenceI):
    """ModelSequence provides an array-based implementation of Sequence.

    Use ModelSequenceBase if you need a stripped-down, fast implementation.
    ModelSequence implements everything that SequenceI implements.

    See docstrings for ModelSequenceBase and SequenceI for information about
    these respective classes.
    """
    def stripBad(self):
        """Returns copy of self with bad chars excised"""
        valid_indices = self._data < len(self.Alphabet)
        result = compress(valid_indices, self._data)
        return self.__class__(result, Info=self.Info)

    def stripBadAndGaps(self):
        """Returns copy of self with bad chars and gaps excised."""
        gap_indices = map(self.Alphabet.index, self.MolType.Gaps)
        valid_indices = self._data < len(self.Alphabet)
        for i in gap_indices:
            valid_indices -= self._data == i
        result = compress(valid_indices, self._data)
        return self.__class__(result, Info=self.Info)

    def stripDegenerate(self):
        """Returns copy of self without degenerate symbols.
        
        NOTE: goes via string intermediate because some of the algorithms
        for resolving degenerates are complex. This could be optimized if
        speed becomes critical.
        """
        return self.__class__(self.MolType.stripDegenerate(str(self)), \
            Info=self.Info)

    def countGaps(self):
        """Returns count of gaps in self."""
        return self.gapArray().sum()

    def gapVector(self):
        """Returns list of bool containing whether each pos is a gap."""
        return map(bool, self.gapArray())

    def gapList(self):
        """Returns list of gap indices."""
        return list(self.gapIndices())

    def gapMaps(self):
        """Returns dicts mapping gapped/ungapped positions."""
        nongaps = logical_not(self.gapArray())
        indices = arange(len(self)).compress(nongaps)
        new_indices = arange(len(indices))
        return dict(zip(new_indices, indices)), dict(zip(indices, new_indices))

    def firstGap(self):
        """Returns position of first gap, or None."""
        a = self.gapIndices()
        try:
            return a[0]
        except IndexError:
            return None

    def isGapped(self):
        """Returns True of sequence contains gaps."""
        return len(self.gapIndices())

    def MW(self, *args, **kwargs):
        """Returns molecular weight.

        Works via string intermediate: could optimize using array of MW if
        speed becomes important.
        """
        return self.MolType.MW(str(self), *args, **kwargs)

    def fracSimilar(self, other, similar_pairs):
        """Returns fraction of positions where self[i] is similar to other[i].
        
        similar_pairs must be a dict such that d[(i,j)] exists if i and j are
        to be counted as similar. Use PairsFromGroups in cogent.util.misc to
        construct such a dict from a list of lists of similar residues.
        
        Truncates at the length of the shorter sequence.
        
        Note: current implementation re-creates the distance function each
        time, so may be expensive compared to creating the distance function
        using for_seq separately.
        
        Returns 0 if one sequence is empty.

        NOTE: goes via string intermediate, could optimize using array if
        speed becomes important. Note that form of similar_pairs input would
        also have to change.
        """
        if not self or not other:
            return 0.0
        
        return for_seq(f = lambda x, y: (x,y) in similar_pairs, \
            normalizer=per_shortest)(str(self), str(other))

class ModelNucleicAcidSequence(ModelSequence):
    """Abstract class defining ops for codons, translation, etc."""
    
    def toCodons(self):
        """Returns copy of self in codon alphabet. Assumes ungapped."""
        alpha_len = len(self.Alphabet)
        return ModelCodonSequence(alpha_len*(\
            alpha_len*self._data[::3] + self._data[1::3]) + self._data[2::3], \
            Name=self.Name, Alphabet=self.Alphabet.Triples)

    def complement(self):
        """Returns complement of sequence"""
        return self.__class__(self.Alphabet._complement_array.take(self._data),\
            Info=self.Info)

    def rc(self):
        """Returns reverse-complement of sequence"""
        comp = self.Alphabet._complement_array.take(self._data)
        return self.__class__(comp[::-1], Info=self.Info)
     
    def toRna(self):
        """Returns self as RNA"""
        return ModelRnaSequence(self._data)

    def toDna(self):
        """Returns self as DNA"""
        return ModelDnaSequence(self._data)


class ModelRnaSequence(ModelNucleicAcidSequence):
    MolType = None  #set to RNA in moltype.py
    Alphabet = None #set to RNA.Alphabets.DegenGapped in moltype.py

    def __init__(self, data='', *args, **kwargs):
        """Returns new ModelRnaSequence, converting T -> U"""
        if hasattr(data, 'upper'):
            data = data.upper().replace('T','U')
        return super(ModelNucleicAcidSequence, self).__init__(data, \
            *args, **kwargs)

class ModelDnaSequence(ModelNucleicAcidSequence):
    MolType = None  #set to DNA in moltype.py
    Alphabet = None #set to DNA.Alphabets.DegenGapped in moltype.py

    def __init__(self, data='', *args, **kwargs):
        """Returns new ModelRnaSequence, converting U -> T"""
        if hasattr(data, 'upper'):
            data = data.upper().replace('U','T')
        return super(ModelNucleicAcidSequence, self).__init__(data, \
            *args, **kwargs)


class ModelCodonSequence(ModelSequence):
    """Abstract base class for codon sequences, incl. string conversion."""
    SequenceClass = ModelNucleicAcidSequence
    def __str__(self):
        """Joins triplets together as string."""
        return self.Delimiter.join(map(''.join, \
            self.Alphabet.fromIndices(self._data)))
    
    def _from_string(self, s):
        """Reads from a raw string, rather than a DnaSequence."""
        s = s.upper().replace('U','T')    #convert to uppercase DNA
        d = self.SequenceClass(s, \
            Alphabet=self.Alphabet.SubEnumerations[0])
        self._data = d.toCodons()._data
    
    def __init__(self, data='', Alphabet=None, Name=None, Info=None):
        """Override __init__ to handle init from string."""
        if isinstance(data, str):
            self._from_string(data)
        ModelSequence.__init__(self, data, Alphabet, Name, Info=Info)
    
    def toCodons(self):
        """Converts self to codons -- in practice, just returns self.
        
        Supports interface of other NucleicAcidSequences."""
        return self
    
    def toDna(self):
        """Returns a ModelDnaSequence from the data in self"""
        unpacked = self.Alphabet.unpackArrays(self._data)
        result = zeros((len(self._data),3))
        for i, v in enumerate(unpacked):
            result[:,i] = v
        return ModelDnaSequence(ravel(result), Name=self.Name)

    def toRna(self):
        """Returns a ModelDnaSequence from the data in self."""
        unpacked = self.Alphabet.unpackArrays(self._data)
        result = zeros((len(self._data),3))
        for i, v in enumerate(unpacked):
            result[:,i] = v
        return ModelRnaSequence(ravel(result), Name=self.Name)

class ModelDnaCodonSequence(ModelCodonSequence):
    """Holds non-degenerate DNA codon sequence."""
    Alphabet = None #set to DNA.Alphabets.Base.Triples in moltype.py
    SequenceClass = ModelDnaSequence

class ModelRnaCodonSequence(ModelCodonSequence):
    """Holds non-degenerate DNA codon sequence."""
    Alphabet = None #set to RNA.Alphabets.Base.Triples in motype.py
    SequenceClass = ModelRnaSequence

    def _from_string(self, s):
        """Reads from a raw string, rather than a DnaSequence."""
        s = s.upper().replace('T','U')    #convert to uppercase DNA
        d = self.SequenceClass(s, \
            Alphabet=self.Alphabet.SubEnumerations[0])
        self._data = d.toCodons()._data

class ModelProteinSequence(ModelSequence):
    MolType = None  #set to PROTEIN in moltype.py
    Alphabet = None #set to PROTEIN.Alphabets.DegenGapped in moltype.py

class ModelProteinWithStopSequence(ModelSequence):
    MolType = None #set to PROTEIN_WITH_STOP in moltype.py
    Alphabet= None #set to PROTEIN_WITH_STOP.Alphabets.DegenGapped in moltype.py
